# coding: utf-8

"""

    No descripton provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

#  (C) Copyright IBM Corp. 2020.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#       http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from pprint import pformat
from six import iteritems
import re


class TrainingStatusExperiments(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, training_guid=None, training_url=None, training_reference_name=None, training_definition_url=None, submitted_at=None, finished_at=None, current_at=None, current_iteration=None, total_iterations=None, state=None, message=None, metrics=None, hyper_parameters=None, result=None, failure=None):
        """
        TrainingStatusExperiments - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'training_guid': 'str',
            'training_url': 'str',
            'training_reference_name': 'str',
            'training_definition_url': 'str',
            'submitted_at': 'datetime',
            'finished_at': 'datetime',
            'current_at': 'datetime',
            'current_iteration': 'float',
            'total_iterations': 'float',
            'state': 'str',
            'message': 'str',
            'metrics': 'ArrayModelVersionMetricsExperiments',
            'hyper_parameters': 'HyperParametersForStatusExperiments',
            'result': 'TrainingStatusExperimentsResult',
            'failure': 'ErrorSchemaExperiments'
        }

        self.attribute_map = {
            'training_guid': 'training_guid',
            'training_url': 'training_url',
            'training_reference_name': 'training_reference_name',
            'training_definition_url': 'training_definition_url',
            'submitted_at': 'submitted_at',
            'finished_at': 'finished_at',
            'current_at': 'current_at',
            'current_iteration': 'current_iteration',
            'total_iterations': 'total_iterations',
            'state': 'state',
            'message': 'message',
            'metrics': 'metrics',
            'hyper_parameters': 'hyper_parameters',
            'result': 'result',
            'failure': 'failure'
        }

        self._training_guid = training_guid
        self._training_url = training_url
        self._training_reference_name = training_reference_name
        self._training_definition_url = training_definition_url
        self._submitted_at = submitted_at
        self._finished_at = finished_at
        self._current_at = current_at
        self._current_iteration = current_iteration
        self._total_iterations = total_iterations
        self._state = state
        self._message = message
        self._metrics = metrics
        self._hyper_parameters = hyper_parameters
        self._result = result
        self._failure = failure

    @property
    def training_guid(self):
        """
        Gets the training_guid of this TrainingStatusExperiments.


        :return: The training_guid of this TrainingStatusExperiments.
        :rtype: str
        """
        return self._training_guid

    @training_guid.setter
    def training_guid(self, training_guid):
        """
        Sets the training_guid of this TrainingStatusExperiments.


        :param training_guid: The training_guid of this TrainingStatusExperiments.
        :type: str
        """

        self._training_guid = training_guid

    @property
    def training_url(self):
        """
        Gets the training_url of this TrainingStatusExperiments.
        Currently, this is link to current /V3 training API, so to the /v3/models.

        :return: The training_url of this TrainingStatusExperiments.
        :rtype: str
        """
        return self._training_url

    @training_url.setter
    def training_url(self, training_url):
        """
        Sets the training_url of this TrainingStatusExperiments.
        Currently, this is link to current /V3 training API, so to the /v3/models.

        :param training_url: The training_url of this TrainingStatusExperiments.
        :type: str
        """

        self._training_url = training_url

    @property
    def training_reference_name(self):
        """
        Gets the training_reference_name of this TrainingStatusExperiments.


        :return: The training_reference_name of this TrainingStatusExperiments.
        :rtype: str
        """
        return self._training_reference_name

    @training_reference_name.setter
    def training_reference_name(self, training_reference_name):
        """
        Sets the training_reference_name of this TrainingStatusExperiments.


        :param training_reference_name: The training_reference_name of this TrainingStatusExperiments.
        :type: str
        """

        self._training_reference_name = training_reference_name

    @property
    def training_definition_url(self):
        """
        Gets the training_definition_url of this TrainingStatusExperiments.


        :return: The training_definition_url of this TrainingStatusExperiments.
        :rtype: str
        """
        return self._training_definition_url

    @training_definition_url.setter
    def training_definition_url(self, training_definition_url):
        """
        Sets the training_definition_url of this TrainingStatusExperiments.


        :param training_definition_url: The training_definition_url of this TrainingStatusExperiments.
        :type: str
        """

        self._training_definition_url = training_definition_url

    @property
    def submitted_at(self):
        """
        Gets the submitted_at of this TrainingStatusExperiments.


        :return: The submitted_at of this TrainingStatusExperiments.
        :rtype: datetime
        """
        return self._submitted_at

    @submitted_at.setter
    def submitted_at(self, submitted_at):
        """
        Sets the submitted_at of this TrainingStatusExperiments.


        :param submitted_at: The submitted_at of this TrainingStatusExperiments.
        :type: datetime
        """

        self._submitted_at = submitted_at

    @property
    def finished_at(self):
        """
        Gets the finished_at of this TrainingStatusExperiments.


        :return: The finished_at of this TrainingStatusExperiments.
        :rtype: datetime
        """
        return self._finished_at

    @finished_at.setter
    def finished_at(self, finished_at):
        """
        Sets the finished_at of this TrainingStatusExperiments.


        :param finished_at: The finished_at of this TrainingStatusExperiments.
        :type: datetime
        """

        self._finished_at = finished_at

    @property
    def current_at(self):
        """
        Gets the current_at of this TrainingStatusExperiments.


        :return: The current_at of this TrainingStatusExperiments.
        :rtype: datetime
        """
        return self._current_at

    @current_at.setter
    def current_at(self, current_at):
        """
        Sets the current_at of this TrainingStatusExperiments.


        :param current_at: The current_at of this TrainingStatusExperiments.
        :type: datetime
        """

        self._current_at = current_at

    @property
    def current_iteration(self):
        """
        Gets the current_iteration of this TrainingStatusExperiments.


        :return: The current_iteration of this TrainingStatusExperiments.
        :rtype: float
        """
        return self._current_iteration

    @current_iteration.setter
    def current_iteration(self, current_iteration):
        """
        Sets the current_iteration of this TrainingStatusExperiments.


        :param current_iteration: The current_iteration of this TrainingStatusExperiments.
        :type: float
        """

        self._current_iteration = current_iteration

    @property
    def total_iterations(self):
        """
        Gets the total_iterations of this TrainingStatusExperiments.


        :return: The total_iterations of this TrainingStatusExperiments.
        :rtype: float
        """
        return self._total_iterations

    @total_iterations.setter
    def total_iterations(self, total_iterations):
        """
        Sets the total_iterations of this TrainingStatusExperiments.


        :param total_iterations: The total_iterations of this TrainingStatusExperiments.
        :type: float
        """

        self._total_iterations = total_iterations

    @property
    def state(self):
        """
        Gets the state of this TrainingStatusExperiments.


        :return: The state of this TrainingStatusExperiments.
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """
        Sets the state of this TrainingStatusExperiments.


        :param state: The state of this TrainingStatusExperiments.
        :type: str
        """
        allowed_values = ["running", "completed", "failed", "canceled"]
        if state not in allowed_values:
            raise ValueError(
                "Invalid value for `state` ({0}), must be one of {1}"
                .format(state, allowed_values)
            )

        self._state = state

    @property
    def message(self):
        """
        Gets the message of this TrainingStatusExperiments.


        :return: The message of this TrainingStatusExperiments.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """
        Sets the message of this TrainingStatusExperiments.


        :param message: The message of this TrainingStatusExperiments.
        :type: str
        """

        self._message = message

    @property
    def metrics(self):
        """
        Gets the metrics of this TrainingStatusExperiments.


        :return: The metrics of this TrainingStatusExperiments.
        :rtype: ArrayModelVersionMetricsExperiments
        """
        return self._metrics

    @metrics.setter
    def metrics(self, metrics):
        """
        Sets the metrics of this TrainingStatusExperiments.


        :param metrics: The metrics of this TrainingStatusExperiments.
        :type: ArrayModelVersionMetricsExperiments
        """

        self._metrics = metrics

    @property
    def hyper_parameters(self):
        """
        Gets the hyper_parameters of this TrainingStatusExperiments.


        :return: The hyper_parameters of this TrainingStatusExperiments.
        :rtype: HyperParametersForStatusExperiments
        """
        return self._hyper_parameters

    @hyper_parameters.setter
    def hyper_parameters(self, hyper_parameters):
        """
        Sets the hyper_parameters of this TrainingStatusExperiments.


        :param hyper_parameters: The hyper_parameters of this TrainingStatusExperiments.
        :type: HyperParametersForStatusExperiments
        """

        self._hyper_parameters = hyper_parameters

    @property
    def result(self):
        """
        Gets the result of this TrainingStatusExperiments.


        :return: The result of this TrainingStatusExperiments.
        :rtype: TrainingStatusExperimentsResult
        """
        return self._result

    @result.setter
    def result(self, result):
        """
        Sets the result of this TrainingStatusExperiments.


        :param result: The result of this TrainingStatusExperiments.
        :type: TrainingStatusExperimentsResult
        """

        self._result = result

    @property
    def failure(self):
        """
        Gets the failure of this TrainingStatusExperiments.


        :return: The failure of this TrainingStatusExperiments.
        :rtype: ErrorSchemaExperiments
        """
        return self._failure

    @failure.setter
    def failure(self, failure):
        """
        Sets the failure of this TrainingStatusExperiments.


        :param failure: The failure of this TrainingStatusExperiments.
        :type: ErrorSchemaExperiments
        """

        self._failure = failure

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



import unittest

import ibm_boto3
from ibm_watson_machine_learning.preprocessing import DataJoinGraph
from ibm_watson_machine_learning.helpers.connections import DataConnection, ContainerLocation, FSLocation
from ibm_watson_machine_learning.tests.utils import is_cp4d, save_multiple_data_to_containers
from ibm_watson_machine_learning.tests.autoai.abstract_tests_classes import AbstractTestOBM, \
    AbstractTestBatch

from ibm_watson_machine_learning.utils.autoai.errors import WrongDataJoinGraphNodeName


from ibm_watson_machine_learning.utils.autoai.enums import PredictionType, Metrics, RegressionAlgorithms


class TestAutoAIRemote(AbstractTestOBM, unittest.TestCase):
    """
    The test can be run on CLOUD, and CPD
    """
    HISTORICAL_RUNS_CHECK = False
    cos_resource = None

    input_data_filenames = ['group_customer_main.csv', 'group_customer_customers.csv', 'group_customer_transactions.csv',
                            'group_customer_purchase.csv', 'group_customer_products.csv']
    input_data_path = './autoai/data/'

    input_node_names = [name.replace('group_customer_', '').split('.')[0] for name in input_data_filenames]

    cos_endpoint = "https://s3.us.cloud-object-storage.appdomain.cloud"
    data_cos_path = "data/"

    SPACE_ONLY = False

    OPTIMIZER_NAME = "Group Customer OBM test sdk"

    target_space_id: str = None

    experiment_info = dict(
        name=OPTIMIZER_NAME,
        desc='test description',
        prediction_type=PredictionType.REGRESSION,
        prediction_column='next_purchase',
        scoring=Metrics.MEDIAN_ABSOLUTE_ERROR,
        holdout_size=0.2,
        include_only_estimators=[RegressionAlgorithms.LGBM, RegressionAlgorithms.XGB]
    )

    def test_00b_write_data_to_container(self):
        if self.SPACE_ONLY:
            self.wml_client.set.default_space(self.space_id)
        else:
            self.wml_client.set.default_project(self.project_id)

        save_multiple_data_to_containers(data_path=self.input_data_path,
                                         data_filenames=self.input_data_filenames,
                                         wml_client=self.wml_client)

    def test_01_create_multiple_data_connections__connections_created(self):
        TestAutoAIRemote.data_connections = []
        for file, node_name in zip(self.input_data_filenames, self.input_node_names):
            conn = DataConnection(
                data_join_node_name=node_name,
                location=ContainerLocation(path=file))
            TestAutoAIRemote.data_connections.append(conn)

        TestAutoAIRemote.results_connection = None

        self.assertIsNotNone(obj=TestAutoAIRemote.data_connections)
        self.assertGreater(len(TestAutoAIRemote.data_connections), 0)
        self.assertIsNone(self.results_connection)

    def test_02_create_data_join_graph__graph_created(self):
        data_join_graph = DataJoinGraph()
        data_join_graph.node(name="main")
        data_join_graph.node(name="customers")
        data_join_graph.node(name="transactions")
        data_join_graph.node(name="purchase")
        data_join_graph.node(name="products")
        data_join_graph.edge(from_node="main", to_node="customers",
                             from_column=["group_customer_id"], to_column=["group_customer_id"])
        data_join_graph.edge(from_node="main", to_node="transactions",
                             from_column=["transaction_id"], to_column=["transaction_id"])
        data_join_graph.edge(from_node="main", to_node="purchase",
                             from_column=["group_id"], to_column=["group_id"])
        data_join_graph.edge(from_node="transactions", to_node="products",
                             from_column=["product_id"], to_column=["product_id"])

        TestAutoAIRemote.data_join_graph = data_join_graph

        print(f"data_join_graph: {data_join_graph}")


    def test_07a_check_data_join_node_names(self):
        data_connections = []
        if self.wml_client.ICP:
            for file in self.input_data_filenames:
                conn = DataConnection(
                    location=FSLocation(path=self.data_location + file))
                data_connections.append(conn)
        else:
            for file in self.input_data_filenames:
                conn = DataConnection(
                    location=ContainerLocation(path=file))
                data_connections.append(conn)

        self.assertGreater(len(data_connections), 0)

        with self.assertRaises(WrongDataJoinGraphNodeName, msg="Training started with wrong data join node names"):
            self.remote_auto_pipelines.fit(
                training_data_reference=data_connections,
                training_results_reference=self.results_connection,
                background_mode=False)

    # def test_08_fit_run_training_of_auto_ai_in_wml(self):
    #
    #     AbstractTestOBM.remote_auto_pipelines = self.experiment.runs.get_optimizer("69d3b663-634b-4157-a114-42d8dc2dc80e")
    #     AbstractTestOBM.trained_pipeline_details = self.remote_auto_pipelines.get_run_details()
    #     AbstractTestOBM.run_id = self.trained_pipeline_details['metadata']['id']
    #
    #     status = self.remote_auto_pipelines.get_run_status()
    #     run_details = self.remote_auto_pipelines.get_run_details().get('entity')
    #     self.assertNotIn(status, ['failed', 'canceled'], msg=f"Training finished with status {status}. \n"
    #                                                          f"Details: {run_details.get('status')}")

if __name__ == '__main__':
    unittest.main()



# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = [
    "WrongDeploymnetType",
    "ModelTypeNotSupported",
    "NotAutoAIExperiment",
    "EnvironmentNotSupported",
    'BatchJobFailed',
    'MissingScoringResults',
    'ModelStoringFailed',
    'DeploymentNotSupported',
    'MissingSpace',
    'ModelPromotionFailed'
]

from ibm_watson_machine_learning.utils import WMLClientError


class WrongDeploymnetType(WMLClientError, ValueError):
    def __init__(self, value_name, reason=None):
        WMLClientError.__init__(self, f"This deployment is not of type: {value_name} ", reason)


class ModelTypeNotSupported(WMLClientError, ValueError):
    def __init__(self, value_name, reason=None):
        WMLClientError.__init__(self, f"This model type is not supported yet: {value_name} ", reason)


class NotAutoAIExperiment(WMLClientError, ValueError):
    def __init__(self, value_name, reason=None):
        WMLClientError.__init__(self, f"This experiment_run_id is not from an AutoAI experiment: {value_name} ", reason)


class EnvironmentNotSupported(WMLClientError, ValueError):
    def __init__(self, value_name, reason=None):
        WMLClientError.__init__(self, f"This environment is not supported: {value_name}", reason)


class BatchJobFailed(WMLClientError, ValueError):
    def __init__(self, value_name=None, reason=None):
        WMLClientError.__init__(self, f"Batch job failed for job: {value_name}", reason)


class MissingScoringResults(WMLClientError, ValueError):
    def __init__(self, value_name=None, reason=None):
        WMLClientError.__init__(self, f"Scoring of deployment job: {value_name} not completed.", reason)


class ModelStoringFailed(WMLClientError, ValueError):
    def __init__(self, value_name=None, reason=None):
        WMLClientError.__init__(self, f"Model: {value_name} store failed.", reason)


class DeploymentNotSupported(WMLClientError, ValueError):
    def __init__(self, value_name=None, reason=None):
        WMLClientError.__init__(self, f"Deployment of type: {value_name} is not supported.", reason)


class MissingSpace(WMLClientError, ValueError):
    def __init__(self, value_name=None, reason=None):
        WMLClientError.__init__(self, f"Deployment needs to have space specified", reason)


class ModelPromotionFailed(WMLClientError, ValueError):
    def __init__(self, project_id, space_id, promotion_response=None, reason=None):
        WMLClientError.__init__(self, f"Model promotion from project {project_id} to space {space_id} failed."
                                      f" Full response {promotion_response}", reason)


"""
IBM Confidential
OCO Source Materials
5737-H76, 5725-W78, 5900-A1R
(c) Copyright IBM Corp. 2020 All Rights Reserved.
The source code for this program is not published or otherwise divested of its trade secrets,
irrespective of what has been deposited with the U.S. Copyright Office.
"""
"""
Module to where data handler are implemented.
"""
import abc

from ibmfl.data.data_handler import DataHandler
from ibmfl.data.env_spec import EnvHandler


class EnvDataHandler(DataHandler):
    """
    Base class to load data and  environment for reinforcement learning.
    """

    @abc.abstractmethod
    def get_data(self, **kwargs):
        """
        Read train data and test data for reinforcement learning
        :return:
        """

    @abc.abstractmethod
    def get_env_class_ref(self) -> EnvHandler:
        """
           Get environment reference for RL trainer, the instance is created in
           model class as part of trainer initialization
        """



# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ibm_watson_machine_learning.wml_client_error import WMLClientError, UnexpectedType
from ibm_watson_machine_learning.wml_resource import WMLResource
from ibm_watson_machine_learning.utils import docstring_parameter, STR_TYPE_NAME


class BothValuesAndRangePassed(WMLClientError):
    def __init__(self):
        WMLClientError.__init__(self, "Both values and range were passed. Only one should be used at the same time.")


class EmptyValues(WMLClientError):
    def __init__(self):
        WMLClientError.__init__(self, "No values were passed.")


class InconsistentValuesTypes(WMLClientError):
    def __init__(self, types):
        WMLClientError.__init__(self, "Passed values have inconsistent types: {}".format(types))


class MissingElementOfRange(WMLClientError):
    def __init__(self, el):
        WMLClientError.__init__(self, "Missing element of range: {}".format(el))


@docstring_parameter({'str_type': STR_TYPE_NAME})
def HPOParameter(name, values=None, max=None, min=None, step=None):
    """
    Prepares dict element describing hyper parameter. Hyper parameter may be contructed in two ways:

    >>> HPOParameter('name', values=['val1', 'val2'])   # values
    >>> HPOParameter('name', min=0.5, max=10, step=0.1) # range

    In range description `min` and `step` are optional (by default `min` is set to 0 and `step` is set to 1).

    :param name: name of parameter
    :type name: {str_type}
    :param values: if parameter should have only values provided by user, this param should contain these values (optional)
    :type values: list of str or list of int or list of float
    :param max: if parameter should contain numbers from range, this will be maximal value of this range (optional)
    :type max: int or float
    :param min: if parameter should contain numbers from range, this will be minimal value of this range (optional)
    :type min: int or float
    :param step: if parameter should contain numbers from range, this will be step between elements from range (optional)
    :type step: int or float

    :return: description of HPO parameter
    :rtype: dict

    A way you might use me is:

    >>> HPOParameter('param1', values=['a', 'b', 'c']),
    >>> HPOParameter('param2', values=[0, 1, 9]),
    >>> HPOParameter('param3', values=[0.1, 0.5, 0.8]),
    >>> HPOParameter('param4', max=10),
    >>> HPOParameter('param5', min=2, max=10),
    >>> HPOParameter('param6', max=10, step=2),
    >>> HPOParameter('param7', max=10.0),
    >>> HPOParameter('param8', min=0.1, max=10),
    >>> HPOParameter('param9', min=0.5, max=10, step=0.1)
    """
    WMLResource._validate_type(name, "name", str, True)
    WMLResource._validate_type(values, "values", list, False)
    WMLResource._validate_type(max, "max", [int, float], False)
    WMLResource._validate_type(min, "min", [int, float], False)
    WMLResource._validate_type(step, "step", [int, float], False)

    if values is not None and (max is not None or min is not None or step is not None):
        raise BothValuesAndRangePassed()
    elif values is not None:
        types = [type(v) for v in values]
        types = list(set(types))

        if len(values) == 0:
            raise EmptyValues()

        if len(types) > 1:
            raise InconsistentValuesTypes(types)

        if types[0] is str:
            return {
                "name": name,
                "string_values": values
            }
        elif types[0] is int:
            return {
                "name": name,
                "int_values": values
            }
        elif types[0] is float:
            return {
                "name": name,
                "double_values": values
            }
        else:
            raise UnexpectedType("values", "str or int or float", types[0])
    elif max is not None:
        if min is None:
            min = 0

        if step is None:
            step = 1

        if type(max) is float or type(min) is float or type(step) is float:
            return {
                "name": name,
                "double_range": {
                    "min_value": min,
                    "max_value": max,
                    "step": step
                }
            }
        else:
            return {
                "name": name,
                "int_range": {
                    "min_value": min,
                    "max_value": max,
                    "step": step
                }
            }
    else:
        raise MissingElementOfRange("max")


def HPOMethodParam(name=None, value=None):
    result = {}

    if name is not None:
        result['name'] = name

    if type(value) == str:
        result['string_value'] = value
    elif type(value) == float:
        result['double_value'] = value
    elif type(value) == int:
        result['int_value'] = value
    else:
        raise UnexpectedType('value', 'str or float or int', type(value))

    return result


import unittest
import copy

from ibm_watson_machine_learning import APIClient

from ibm_watson_machine_learning.tests.utils import (get_wml_credentials, is_cp4d)
from ibm_watson_machine_learning.wml_client_error import WMLClientError, CannotAutogenerateBedrockUrl


@unittest.skipIf(not is_cp4d(), "Not supported on cloud")
class TestAutoAIRemote(unittest.TestCase):
    """
    The test can be run on CLOUD, WMLS and CPD (not tested)
    The test covers:
    - COS set-up (if run on Cloud): checking if bucket exists for the cos instance, if not new bucket is create
    - Saving data `/bank.cdv` to COS/data assets
    - downloading training data from cos/data assets
    - downloading all generated pipelines to lale pipeline
    - deployment with lale pipeline
    - deployment deletion
    """
    wml_credentials = None
    token = None

    @classmethod
    def setUpClass(cls) -> None:
        """
        Load WML credentials from config.ini file based on ENV variable.
        """
        cls.wml_credentials = get_wml_credentials()
        wml_client = APIClient(wml_credentials=cls.wml_credentials)
        cls.token = wml_client.wml_token

    def test_01_missing_version(self):
        wml_credentials = copy.copy(self.wml_credentials)
        del wml_credentials['version']

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('Missing version in credentials for Cloud Pak for Data. Add key `version` to your credentials with a supported value, referring to your software version' in context.exception.error_msg)

    def test_02_missing_url(self):
        wml_credentials = copy.copy(self.wml_credentials)
        del wml_credentials['url']

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('`url` is not provided' in context.exception.error_msg)

    def test_03_missing_instance_id(self):
        wml_credentials = copy.copy(self.wml_credentials)
        del wml_credentials['instance_id']

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('The specified url is not valid. To authenticate with your Cloud Pak for Data installed software, add `"instance_id": "openshift"` to your credentials. To authenticate with your Cloud Pak for Data as a Service account, see https://dataplatform.cloud.ibm.com/docs/content/wsj/analyze-data/ml-authentication.html.' in context.exception.error_msg)

    def test_04_invalid_version(self):
        wml_credentials = copy.copy(self.wml_credentials)
        wml_credentials['version'] = 'banana'

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('Invalid value for \'version\' provided in wml_credentials' in context.exception.error_msg)

    def test_05_invalid_url(self):
        wml_credentials = copy.copy(self.wml_credentials)
        wml_credentials['url'] = 'banana'

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('`url` must start with `https://`.' in context.exception.error_msg)

    def test_06_invalid_instance_id(self):
        wml_credentials = copy.copy(self.wml_credentials)
        wml_credentials['instance_id'] = 'banana'

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('Invalid instance_id for Cloud Pak for Data. Use `"instance_id": "openshift"` in your credentials. To authenticate with a different offering, refer to the product documentation for authentication details.' in context.exception.error_msg)

    def test_username_password_auth_scenario_01_correct(self):
        wml_credentials = {
            'instance_id': self.wml_credentials['instance_id'],
            'url': self.wml_credentials['url'],
            'version': self.wml_credentials['version'],
            'username': self.wml_credentials['username'],
            'password': self.wml_credentials['password']
        }
        APIClient(wml_credentials=wml_credentials)

    def test_username_password_auth_scenario_02_missing_password(self):
        wml_credentials = {
            'instance_id': self.wml_credentials['instance_id'],
            'url': self.wml_credentials['url'],
            'version': self.wml_credentials['version'],
            'username': self.wml_credentials['username']
        }

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('`password` missing in wml_credentials.' in context.exception.error_msg)

    def test_username_password_auth_scenario_03_missing_username(self):
        wml_credentials = {
            'instance_id': self.wml_credentials['instance_id'],
            'url': self.wml_credentials['url'],
            'version': self.wml_credentials['version'],
            'password': self.wml_credentials['password']
        }

        with self.assertRaises(CannotAutogenerateBedrockUrl) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('`username` missing in wml_credentials.' in context.exception.args[0].error_msg)

    def test_username_apikey_auth_scenario_01_correct(self):
        if 'apikey' not in self.wml_credentials:
            self.skipTest("No apikey in creds")

        wml_credentials = {
            'instance_id': self.wml_credentials['instance_id'],
            'url': self.wml_credentials['url'],
            'version': self.wml_credentials['version'],
            'username': self.wml_credentials['username'],
            'apikey': self.wml_credentials['apikey']
        }
        APIClient(wml_credentials=wml_credentials)

    def test_username_apikey_auth_scenario_02_invalid_apikey_key(self):
        if 'apikey' not in self.wml_credentials:
            self.skipTest("No apikey in creds")

        wml_credentials = {
            'instance_id': self.wml_credentials['instance_id'],
            'url': self.wml_credentials['url'],
            'version': self.wml_credentials['version'],
            'username': self.wml_credentials['username'],
            'apikey': self.wml_credentials['apikey']
        }
        APIClient(wml_credentials=wml_credentials)

    def test_token_auth_scenario_01_correct(self):
        wml_credentials = {
            'instance_id': self.wml_credentials['instance_id'],
            'url': self.wml_credentials['url'],
            'version': self.wml_credentials['version'],
            'token': self.token
        }
        APIClient(wml_credentials=wml_credentials)

    # For CPD it cannot be determined at this stage
    # def test_token_auth_scenario_02_invalid_token(self):
    #     wml_credentials = {
    #         'instance_id': self.wml_credentials['instance_id'],
    #         'url': self.wml_credentials['url'],
    #         'version': self.wml_credentials['version'],
    #         'token': 'banana'
    #     }
    #     with self.assertRaises(WMLClientError) as context:
    #         APIClient(wml_credentials=wml_credentials)
    #
    #     self.assertTrue('invalid token' in context.exception.error_msg)

    def test_token_auth_scenario_03_missing_token(self):
        wml_credentials = {
            'instance_id': self.wml_credentials['instance_id'],
            'url': self.wml_credentials['url'],
            'version': self.wml_credentials['version']
        }

        with self.assertRaises(WMLClientError) as context:
            APIClient(wml_credentials=wml_credentials)

        self.assertTrue('`username` missing in wml_credentials.' in context.exception.error_msg)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
Copyright 2016 SmartBear Software

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Ref: https://github.com/swagger-api/swagger-codegen
"""

#  (C) Copyright IBM Corp. 2020.
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#       http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from pprint import pformat
from six import iteritems


class MlAssetsCreateModelInput(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self,
                 tags=None,
                 space=None,
                 project=None,
                 custom=None,
                 pipeline=None,
                 training_lib=None,
                 type=None,
                 domain=None,
                 name=None,
                 description=None,
                 transformed_label_column=None,
                 label_column=None,
                 training_data_references=None,
                 import_location=None,
                 size=None,
                 hyper_parameters=None,
                 runtime=None,
                 metrics=None,
                 schemas=None,
                 software_spec=None,
                 model_definition=None,
                 project_id=None,
                 space_id=None,
                 ):
        """
        MlAssetsCreateModelInput - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'tags': 'list[TagRepository]',
            'space': 'ModelSpaces',
            'project': 'SpaceModels',
            'custom': 'ModelsCustom',
            'pipeline': 'PipelinesModels',
            'training_lib': 'TrainingModels',
            'type': 'str',
            'domain': 'str',
            'name': 'str',
            'description': 'str',
            'transformed_label_column': 'str',
            'label_column': 'str',
            'training_data_references': 'list[ModelContentLocation]',
            'import_location': 'ModelContentLocation',
            'size': 'ModelsSize',
            'hyper_parameters':'list[HyperParameters]',
            'runtime':'RuntimeModels',
            'metrics': 'ModelsMetrics',
            'schemas': 'ModelSchemas',
            'software_spec': 'SoftwareSpecModels',
            'model_definition': 'ModelDefinitionModels',
            'project_id': 'ProjectId',
            'space_id': 'SpaceId'

        }

        self.attribute_map = {
            'tags': 'tags',
            'space': 'space',
            'project': 'project',
            'custom': 'custom',
            'pipeline': 'pipeline',
            'training_lib': 'training_lib',
            'type': 'type',
            'domain': 'domain',
            'name': 'name',
            'description': 'description',
            'transformed_label_column': 'transformed_label_column',
            'label_column': 'label_column',
            'training_data_references': 'training_data_references',
            'import_location': 'import',
            'size': 'size',
            'hyper_parameters':'hyper_parameters',
            'runtime':'runtime',
            'metrics': 'metrics',
            'schemas': 'schemas',
            'software_spec': 'software_spec',
            'model_definition': 'model_definition',
            'project_id':'project_id',
            'space_id':'space_id'
        }

        self._tags = tags
        self._name = name
        self._description = description
        self._transformed_label_column = transformed_label_column
        self._label_column = label_column
        self._training_data_references = training_data_references
        self._schemas = schemas
        self._hyper_parameters = hyper_parameters
        self._runtime = runtime
        self._space = space
        self._project = project
        self._custom = custom
        self._pipeline =  pipeline
        self._training_lib =  training_lib
        self._type = type
        self._domain = domain
        self._import_location = import_location
        self._metrics = metrics
        self._size = size
        self._software_spec = software_spec
        self._model_definition = model_definition
        self._project_id = project_id
        self._space_id = space_id

    @property
    def tags(self):
        """
        Gets the tags of this ExperimentInputExperiments.


        :return: The tags of this ExperimentInputExperiments.
        :rtype: list[TagRepository]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """
        Sets the tags of this ExperimentInputExperiments.


        :param tags: The tags of this ExperimentInputExperiments.
        :type: list[TagRepository]
        """

        self._tags = tags

    @property
    def sofrware_spec(self):
        """
        Gets the software_spec of this Model


        :return: The software_spec of this models.
        :rtype: SoftwareSpecModels
        """
        return self._software_spec

    @sofrware_spec.setter
    def software_spec(self, software_spec):
        """
        Sets the software_spec of this Models.


        :param software_spec: The software_spec of this Models.
        :type: SoftwareSpecModels
        """

        self._software_spec = software_spec

    @property
    def model_definition(self):
        """
        Gets the model_definition of this Model


        :return: The model_definition of this models.
        :rtype: ModelDefinitionModels
        """
        return self._model_definition

    @model_definition.setter
    def model_definition(self, model_definition):
        """
        Sets the model_definition of this Model.


        :param model_definition: The model_definition of this Model.
        :type: ModelDefinitionModels
        """

        self._model_definition = model_definition

    @property
    def project_id(self):
        """
        Gets the project_id of this Model


        :return: The project id of this models.
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """
        Sets the project_id of this Model input.


        :param project_id: The project_id of this Modelinput
        :type: str
        """

        self._project_id = project_id

    @property
    def space_id(self):
        """
        Gets the space_id of this Model


        :return: The space id of this models.
        :rtype: str
        """
        return self._space_id

    @space_id.setter
    def space_id(self, space_id):
        """
        Sets the space_id of this Model input.


        :param space_id: The space_id of this Modelinput
        :type: str
        """

        self._space_id = space_id

    @property
    def hyper_parameters(self):
        """
        Gets the hyper_parameters of this MlAssetsCreateModelInput.


        :return: The hyper_parameters of this MlAssetsCreateModelInput.
        :rtype: HyperParameters
        """
        return self._hyper_parameters

    @hyper_parameters.setter
    def hyper_parameters(self, hyper_parameters):
        """
        Sets the framework of this MlAssetsCreateModelInput.


        :param framework: The framework of this MlAssetsCreateModelInput.
        :type: HyperParameters
        """
        self._hyper_parameters = hyper_parameters

    @property
    def custom(self):
        """
        Gets the custom of this MlAssetsCreateModelInput.


        :return: The custom of this MlAssetsCreateModelInput.
        :rtype: custom
        """
        return self._custom

    @custom.setter
    def custom(self, custom):
        """
        Sets the framework of this MlAssetsCreateModelInput.


        :param framework: The framework of this MlAssetsCreateModelInput.
        :type: custom
        """
        self._custom = custom

    @property
    def schemas(self):
        """
        Gets the hyper_parameters of this MlAssetsCreateModelInput.


        :return: The hyper_parameters of this MlAssetsCreateModelInput.
        :rtype: HyperParameters
        """
        return self._schemas

    @schemas.setter
    def schemas(self, schemas):
        """
        Sets the framework of this MlAssetsCreateModelInput.


        :param framework: The framework of this MlAssetsCreateModelInput.
        :type: HyperParameters
        """
        self._schemas = schemas

    @property
    def name(self):
        """
        Gets the name of this MlAssetsCreateModelInput.


        :return: The name of this MlAssetsCreateModelInput.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this MlAssetsCreateModelInput.


        :param name: The name of this MlAssetsCreateModelInput.
        :type: str
        """
        self._name = name

    @property
    def description(self):
        """
        Gets the description of this MlAssetsCreateModelInput.


        :return: The description of this MlAssetsCreateModelInput.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this MlAssetsCreateModelInput.


        :param description: The description of this MlAssetsCreateModelInput.
        :type: str
        """
        self._description = description



    @property
    def transformed_label_column(self):
        """
        Gets the transformed_label_column of this MlAssetsCreateModelInput.


        :return: The transformed_label_column of this MlAssetsCreateModelInput.
        :rtype: str
        """
        return self._transformed_label_column

    @transformed_label_column.setter
    def transformed_label_column(self, transformed_label_column):
        """
        Sets the transformed_label_column of this MlAssetsCreateModelInput.


        :param transformed_label_column: The transformed_label_column of this MlAssetsCreateModelInput.
        :type: str
        """
        self._transformed_label_column = transformed_label_column



    @property
    def label_column(self):
        """
        Gets the label_column of this MlAssetsCreateModelInput.
        Name of the label column

        :return: The label_column of this MlAssetsCreateModelInput.
        :rtype: str
        """
        return self._label_column

    @label_column.setter
    def label_column(self, label_column):
        """
        Sets the label_column of this MlAssetsCreateModelInput.
        Name of the label column

        :param label_column: The label_column of this MlAssetsCreateModelInput.
        :type: str
        """
        self._label_column = label_column


    @property
    def training_data_references(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._training_data_references


    @training_data_references.setter
    def training_data_reference(self, training_data_references):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._training_data_references = training_data_references


    @property
    def evaluation(self):
        """
        Gets the evaluation of this MlAssetsCreateModelInput.
        Definition of the evaluation method used for pipeline models

        :return: The evaluation of this MlAssetsCreateModelInput.
        :rtype: EvaluationDefinitionRepository
        """
        return self._evaluation

    @evaluation.setter
    def evaluation(self, evaluation):
        """
        Sets the evaluation of this MlAssetsCreateModelInput.
        Definition of the evaluation method used for pipeline models

        :param evaluation: The evaluation of this MlAssetsCreateModelInput.
        :type: EvaluationDefinitionRepository
        """
        self._evaluation = evaluation


    @property
    def runtime(self):
        """
        Gets the runtime of this MlAssetsCreateModelInput.


        :return: The runtime of this MlAssetsCreateModelInput.
        :rtype: runtime
        """
        return self._runtime

    @runtime.setter
    def runtime(self, runtime):
        """
        Sets the runtime of this MlAssetsCreateModelInput.

        :param runtime: The runtime of this MlAssetsCreateModelInput.
        :type: runtime
        """
        self._runtime = runtime



    @property
    def space(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._space

    @space.setter
    def space(self, space):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._space = space

    @property
    def project(self):
        """
        Gets the project of this MlAssetsCreateModelInput.
        :return: The project of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Sets the project of this MlAssetsCreateModelInput.
        :param project: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._project = project

    @property
    def pipeline(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._pipeline

    @pipeline.setter
    def pipeline(self, pipeline):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._pipeline = pipeline



    @property
    def training_lib(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._training_lib

    @training_lib.setter
    def training_lib(self, training_lib):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._training_lib = training_lib

    @property
    def type(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._type = type

    @property
    def domain(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._domain

    @domain.setter
    def domain(self, domain):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._domain = domain

    @property
    def metrics(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._metrics

    @metrics.setter
    def metrics(self, metrics):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._metrics = metrics

    @property
    def import_location(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._import_location

    @import_location.setter
    def import_location(self, import_location):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._import_location = import_location

    @property
    def size(self):
        """
        Gets the training_data of this MlAssetsCreateModelInput.
        :return: The training_data of this MlAssetsCreateModelInput.
        :rtype: ArrayDataInputRepository
        """
        return self._size

    @size.setter
    def size(self, size):
        """
        Sets the training_data of this MlAssetsCreateModelInput.
        :param training_data: The training_data of this MlAssetsCreateModelInput.
        :type: ArrayDataInputRepository
        """
        self._size = size



    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                if (attr == "import_location"):
                    result["import"] =  dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
                else:
                    result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                if (attr == "import_location"):
                    result["import"] = value
                else:
                    result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other




import unittest

from ibm_watson_machine_learning.helpers import DataConnection, CP4DAssetLocation, FSLocation, ConnectionAsset, ConnectionAssetLocation
from ibm_watson_machine_learning.tests.utils import get_cos_credentials, get_wml_credentials, print_test_separators
from ibm_watson_machine_learning.experiment import AutoAI
from ibm_watson_machine_learning.experiment.autoai.optimizers.local_auto_pipelines import LocalAutoPipelines
from lale.operators import TrainedPipeline
from ibm_watson_machine_learning.deployment import WebService
from ibm_watson_machine_learning.utils.autoai.enums import PipelineTypes
from sklearn.pipeline import Pipeline
import pandas as pd


class TestLocalOptimizerCPD(unittest.TestCase):
    path = 'bank.csv'
    pipeline_model_auth_endpoint = None
    metadata = None
    metadata_auth_endpoint = None
    pipeline_name = 'Pipeline_1'
    # training_status = 'auto_ml/4f4da88c-3a16-4450-9d3c-7bb3017164ae/wml_data/cf660105-7d43-480c-8ea1-b3d532386ab5/training-status.json'
    training_status = 'auto_ml/auto_ml.4392c2a1-4270-4b01-84f1-80a9b598e5a3/wml_data/00727c93-9fd3-4b5f-bc1a-8223ef4ac1ac/training-status.json'
    # model_location = 'auto_ml/4f4da88c-3a16-4450-9d3c-7bb3017164ae/wml_data/cf660105-7d43-480c-8ea1-b3d532386ab5/data/automl/global_output/Pipeline0/model.pickle'
    model_location = 'auto_ml/auto_ml.4392c2a1-4270-4b01-84f1-80a9b598e5a3/wml_data/00727c93-9fd3-4b5f-bc1a-8223ef4ac1ac/data/automl/global_output/Pipeline0/model.pickle'

    local_optimizer: 'LocalAutoPipelines' = None
    connection_id = '66d1d6c9-aa49-4583-b020-f1d7eaf416c1'
    project_id = 'acdee0a5-104d-497e-b6bd-d37601e072f4'
    space_id = 'c6f02cdc-ca64-4285-b248-b5ad788be456'

    data_location = './autoai/data/bank.csv'
    data = None
    X = None
    y = None

    @classmethod
    def setUp(cls) -> None:
        cls.wml_credentials = get_wml_credentials()
        cls.data = pd.read_csv(cls.data_location)
        cls.X = cls.data.drop(['y'], axis=1)
        cls.y = cls.data['y']

    @print_test_separators
    def test_01a_get_optimizer(self):
        print("Initializing DataConnections...")

        # training_data_reference = [DataConnection(
        #     location=CP4DAssetLocation(asset_id=self.asset_id)
        # )]

        training_data_reference = [DataConnection(
            connection=ConnectionAsset(connection_id='66d1d6c9-aa49-4583-b020-f1d7eaf416c1'),
            location=ConnectionAssetLocation(bucket='somebucket2', file_name='bank.csv'))]

        training_result_reference = DataConnection(
            location=FSLocation(
                # path='projects/4ae1998c-87be-49df-9baf-c79ce3edbd4b/assets/auto_ml/auto_ml.190f5e37-59f3-4d95-9749-a49b6caf6da8/wml_data/4667839f-ae61-419d-a42c-adb6b70df380/data/automl',
                path='/projects/acdee0a5-104d-497e-b6bd-d37601e072f4/assets/auto_ml/auto_ml.4392c2a1-4270-4b01-84f1-80a9b598e5a3/wml_data/00727c93-9fd3-4b5f-bc1a-8223ef4ac1ac/data/automl',
            ))

        TestLocalOptimizerCPD.metadata = dict(
            training_data_reference=training_data_reference,
            training_result_reference=training_result_reference,
            prediction_type='classification',
            prediction_column='y',
            test_size=0.2,
            scoring='roc_auc',
            max_number_of_estimators=1,
        )

        print("Initializing AutoAI local scenario with metadata...")

        TestLocalOptimizerCPD.local_optimizer = AutoAI(
            wml_credentials=self.wml_credentials,
            project_id=self.project_id
        ).runs.get_optimizer(
            metadata=self.metadata
        )

    @print_test_separators
    def test_02_get_pipeline(self):
        print("AUTH: Fetching and store pipeline by name, LALE...")

        pipeline_lale = self.local_optimizer.get_pipeline(pipeline_name=self.pipeline_name, persist=True)
        self.assertIsInstance(pipeline_lale, TrainedPipeline, msg="Loaded model should be of type TrainedPipeline")

        print("AUTH: Fetching pipeline by name, SKLEARN...")

        pipeline_skl = self.local_optimizer.get_pipeline(pipeline_name=self.pipeline_name, astype=PipelineTypes.SKLEARN)
        self.assertIsInstance(pipeline_skl, Pipeline, msg="Loaded model should be of type SKLEARN")

        print("AUTH: Fetching best pipeline, LALE...")

        try:
            TestLocalOptimizerCPD.pipeline_model_auth_endpoint = self.local_optimizer.get_pipeline()
        except Exception as e:
            self.assertIsInstance(self.pipeline_model_auth_endpoint, TrainedPipeline,
                                  msg=f"Cannot load best pipeline model, Error: {e}")

    @print_test_separators
    def test_03_get_params__fetch_dict_with_parameters(self):
        print("AUTH: Getting optimizer parameters...")

        params = self.local_optimizer.get_params()
        print(params)

    @print_test_separators
    def test_04_fit__not_implemented(self):
        print("AUTH: Calling fit() method...")

        with self.assertRaises(NotImplementedError, msg="\"fit\" should raise NotImplementedError"):
            self.local_optimizer.fit(X=self.X, y=self.y)

    @print_test_separators
    def test_05_summary__return_data_frame_with_summary(self):
        print("AUTH: Fetching summary of experiment...")
        summary = self.local_optimizer.summary()
        print(summary)

    @print_test_separators
    def test_06_get_pipeline_details(self):
        print("AUTH: Fetching best pipeline details...")

        best_pipeline_details = self.local_optimizer.get_pipeline_details()
        print(f"best pipeline details: {best_pipeline_details}")

        print("AUTH: Fetching pipeline details...")

        pipeline_details = self.local_optimizer.get_pipeline_details(pipeline_name=self.pipeline_name)
        print(f"pipeline details: {pipeline_details}")

    @print_test_separators
    def test_07_predict__on_best_pipeline(self):
        print("AUTH: Calling predict on the best pipeline...")

        predictions = self.local_optimizer.predict(X=self.X)
        print(predictions)

    @print_test_separators
    def test_08_get_data_connections__return_training_data_connection_and_recreate_holdout_split(self):
        print("AUTH: Reading training data with holdout split...")

        training_df, holdout_df = self.local_optimizer.get_data_connections()[0].read(with_holdout_split=True)
        print(f"Training data frame: {training_df}")
        print(f"Holdout data frame: {holdout_df}")

    @print_test_separators
    def test_09_deploy_model_from_object(self):
        print("AUTH: Deploying model from object...")

        service = WebService(
            source_wml_credentials=self.wml_credentials,
            source_project_id=self.project_id,
            target_wml_credentials=self.wml_credentials,
            target_space_id=self.space_id)

        service.create(
            model=self.pipeline_model_auth_endpoint,
            metadata=self.metadata,
            deployment_name="Test deployment from auto-gen notebook"
        )

        print(service)
        print(service.get_params())
        predictions = service.score(payload=self.X.iloc[:10])
        print(predictions)
        print(service.list())
        print(service.delete())
        print(service.list())

    @print_test_separators
    def test_10_deploy_model_from_pipeline_name(self):
        print("AUTH: Deploying model from pipeline name...")

        service = WebService(
            source_wml_credentials=self.wml_credentials,
            source_project_id=self.project_id,
            target_wml_credentials=self.wml_credentials,
            target_space_id=self.space_id)

        service.create(
            model=self.pipeline_name,
            metadata=self.metadata,
            deployment_name="Test deployment from auto-gen notebookS"
        )

        print(service)
        print(service.get_params())
        predictions = service.score(payload=self.X.iloc[:10])
        print(predictions)
        print(service.list())
        print(service.delete())
        print(service.list())


if __name__ == '__main__':
    unittest.main()



# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import re

from ibm_watson_machine_learning.libs.repo.mlrepository import  ModelArtifact
from ibm_watson_machine_learning.libs.repo.mlrepository import MetaNames, MetaProps
from ibm_watson_machine_learning.libs.repo.mlrepository.scikit_model_artifact import ScikitModelArtifact
from ibm_watson_machine_learning.libs.repo.mlrepository.tensorflow_model_artifact import TensorflowModelArtifact
from ibm_watson_machine_learning.libs.repo.mlrepository.generic_archive_model_artifact import GenericArchiveModelArtifact
from ibm_watson_machine_learning.libs.repo.mlrepositoryartifact import SparkPipelineModelLoader, SparkPipelineModelContentLoader,\
    IBMSparkPipelineModelContentLoader, MLPipelineModelContentLoader
from ibm_watson_machine_learning.libs.repo.mlrepositoryartifact.content_loaders import ScikitPipelineModelContentLoader,TensorflowPipelineModelContentLoader
from ibm_watson_machine_learning.libs.repo.mlrepositoryartifact.scikit_pipeline_model_loader import ScikitPipelineModelLoader
from ibm_watson_machine_learning.libs.repo.mlrepositoryartifact.tensorflow_pipeline_model_loader import TensorflowPipelineModelLoader
from ibm_watson_machine_learning.libs.repo.mlrepositoryartifact.generic_file_pipeline_model_loader import GenericFilePipelineModelLoader
from ibm_watson_machine_learning.libs.repo.mlrepositoryartifact.hybrid_pipeline_model_artifact import HybridPipelineModelArtifact
from ibm_watson_machine_learning.libs.repo.mlrepositoryartifact.hybrid_pipeline_model_loader import  HybridPipelineModelLoader
from ibm_watson_machine_learning.libs.repo.swagger_client.models import AuthorRepository
from ibm_watson_machine_learning.libs.repo.util.generic_archive_file_check import GenericArchiveFrameworkCheck
import ast,json

from ibm_watson_machine_learning.libs.repo.util.library_imports import LibraryChecker
from ibm_watson_machine_learning.libs.repo.base_constants import *

lib_checker = LibraryChecker()

logger = logging.getLogger('ModelAdapter')


class ModelAdapter(object):
    """
    Adapter creating pipeline model artifact using output from service.
    """

    @staticmethod
    def __strip_output(output):
        return ast.literal_eval(json.dumps(output))

    def __init__(self, model_output, model_version_output, client):
        self.model_output = model_output
        self.model_version_output = model_version_output
        self.client = client
        self.model_type = model_output.entity['type']
        self.model_entity = model_output.entity
        self.model_metadata = model_output.metadata

    def artifact(self):
        if re.match('mllib', self.model_type) is not None or 'mllib' in self.model_type:
            lib_checker.check_lib(PYSPARK)
            model_artifact_builder = type(
                "ModelArtifact",
                (SparkPipelineModelContentLoader, SparkPipelineModelLoader, ModelArtifact, object),
                {}
            )
        elif re.match('wml_1.', self.model_type) is not None or 'wml_1.' in self.model_type:
            lib_checker.check_lib(PYSPARK)
            lib_checker.check_lib(MLPIPELINE)
            model_artifact_builder = type(
                "WMLModelArtifact",
                (MLPipelineModelContentLoader, SparkPipelineModelLoader, ModelArtifact, object),
                {}
            )
        elif re.match('scikit-learn', self.model_type) is not None or 'scikit-learn' in self.model_type:
            lib_checker.check_lib(SCIKIT)
            model_artifact_builder = type(
                "ScikitModelArtifact",
                (ScikitPipelineModelContentLoader, ScikitPipelineModelLoader, ScikitModelArtifact, object),
                {}
            )
        elif re.match('tensorflow', self.model_type) is not None or 'tensorflow' in self.model_type:
            lib_checker.check_lib(TENSORFLOW)
            model_artifact_builder = type(
                "TensorflowModelArtifact",
                (TensorflowPipelineModelContentLoader, TensorflowPipelineModelLoader, TensorflowModelArtifact, object),
                {}
            )

        elif re.match('xgboost', self.model_type) is not None or 'xgboost' in self.model_type:
            lib_checker.check_lib(SCIKIT)
            lib_checker.check_lib(XGBOOST)
            model_artifact_builder = type(
                "ScikitModelArtifact",
                (ScikitPipelineModelContentLoader, ScikitPipelineModelLoader, ScikitModelArtifact, object),
                {}
            )
        elif re.match('hybrid', self.model_type) is not None or \
             re.match('wml-hybrid', self.model_type) is not None or \
             'hybrid' in  self.model_type or 'wml-hybrid' in self.model_type:
            model_artifact_builder = type(
                "HybridModelArtifact",
                (HybridPipelineModelLoader, HybridPipelineModelArtifact,object),
                {}
            )
        elif GenericArchiveFrameworkCheck.is_archive_framework(self.model_type):
            model_artifact_builder = type(
                "GenericModelArchiveArtifact",
                (GenericFilePipelineModelLoader, GenericArchiveModelArtifact, object),
                {}
            )
        else:
            if isinstance(self.model_type, str):
                raise ValueError('Invalid model_type: {}'.format(self.model_type))
            else:
                raise ValueError('Invalid model_type: {}'.format(self.model_type.get('name')))

        prop_map = {
            MetaNames.TYPE: self.model_type
        }

        if self.model_metadata.created_at is not None:
            prop_map[MetaNames.CREATION_TIME] = self.model_output.metadata.created_at

        if self.model_output.metadata.modified_at is not None:
            prop_map[MetaNames.LAST_UPDATED] = self.model_output.metadata.modified_at

        if self.model_entity.get('domain', None) is not None:
            prop_map[MetaNames.DOMAIN] = self.model_entity.get('domain')

        if self.model_entity.get('custom', None) is not None:
            prop_map[MetaNames.CUSTOM] = self.model_entity.get('custom')

        if self.model_entity.get('label_column', None) is not None:
            prop_map[MetaNames.LABEL_FIELD] = self.model_entity.get('label_column')

        if self.model_entity.get('description', None) is not None:
            prop_map[MetaNames.DESCRIPTION] = self.model_entity.get('description')

        if self.model_entity.get('transformed_label', None) is not None:
            prop_map[MetaNames.TRANSFORMED_LABEL_FIELD] = self.model_entity.get('transformed_label')

        if self.model_entity.get('tags', None) is not None:
            prop_map[MetaNames.TAGS] = self.model_entity.get('tags')

        if self.model_entity.get('size', None) is not None:
            prop_map[MetaNames.SIZE] = self.model_entity.get('size')

        if "training_data_references" in self.model_entity and \
                self.model_entity.get('training_data_references', None) is not None:
            prop_map[MetaNames.TRAINING_DATA_REFERENCES] = self.model_entity.get('training_data_references')


        #v4 cloud specific model artifact:

        href_tmp = getattr(self.model_metadata, 'href', 'None')

        if self.model_metadata.id is not None and href_tmp is None:
            if self.model_entity.get('pipeline') is not None:
                runtimeval = self.model_entity.get('pipeline')
                if runtimeval['id'] is not None:
                    prop_map[MetaNames.PIPELINE_UID] = self.model_entity.get('pipeline')['id']

            if self.model_entity.get('model_definition') is not None:
                runtimeval = self.model_entity.get('model_definition')
                if runtimeval['id'] is not None:
                    prop_map[MetaNames.model_definition] = self.model_entity.get('model_definition')['id']

            if self.model_metadata.space_id is not None:
                prop_map[MetaNames.SPACE_ID] = self.model_metadata.space_id

            if self.model_metadata.project_id is not None:
                prop_map[MetaNames.PROJECT_ID] = self.model_metadata.project_id

            if self.model_entity.get('hyper_parameters') is not None:
                prop_map.add(MetaNames.HYPER_PARAMETERS, self.model_entity['hyper_parameters'])

            if self.model_entity.get('software_spec') is not None:
                runtimeval = self.model_entity.get('software_spec')
                if runtimeval['id'] is not None:
                    prop_map[MetaNames.SOFTWARE_SPEC] = self.model_entity.get('software_spec')['id']
            if self.model_entity.get('schemas') is not None and self.model_entity['schemas']['input'] is not None:
                prop_map[MetaNames.INPUT_DATA_SCHEMA] = self.model_entity['schemas']['input']

            if self.model_entity.get('schemas') is not None and self.model_entity['schemas']['output'] is not None:
                prop_map[MetaNames.OUTPUT_DATA_SCHEMA] = self.model_entity['schemas']['output']

            if 'metrics' in self.model_entity and self.model_entity['metrics'] is not None:
                prop_map[MetaNames.METRICS] = self.model_entity['metrics']

            if 'content_status' in self.model_entity:
                if self.model_entity['content_status'] is not None:
                    prop_map[MetaNames.CONTENT_STATUS] = self.model_entity['content_status']

            #TODO : Enable this, when we support import
            # if 'import' in self.model_entity:
            #     if self.model_entity['import'] is not None:
            #         prop_map.add(MetaNames.IMPORT, self.model_entity['import'])

            name = self.model_metadata.name
            model_id = self.model_metadata.id

            model_artifact = model_artifact_builder(
                uid=model_id,
                name=name,
                meta_props=MetaProps(prop_map)
            )

            model_artifact.client = self.client
            model_artifact._content_href = "/ml/v4/models/" + self.model_metadata.id + "/content"
            model_artifact._download_href = "/ml/v4/models/" + self.model_metadata.id + \
                                            "/download?version=2021-06-24"
            # TODO Don't use hardcoded version.
            # ( self.client.version_param <- this is not APIClient it is MLRepositoryClient)
        else:
            if self.model_entity.get('runtime') is not None:
                runtimeval = self.model_entity.get('runtime')
                if runtimeval['href'] is not None:
                    prop_map[MetaNames.RUNTIMES] = self.model_entity.get('runtime')['href']

            if self.model_entity.get('pipeline') is not None:
                runtimeval = self.model_entity.get('pipeline')
                if runtimeval['href'] is not None:
                    prop_map[MetaNames.PIPELINE_UID] = self.model_entity.get('pipeline')['href']

            if self.model_entity.get('training_lib') is not None:
                runtimeval = self.model_entity.get('training_lib')
                if runtimeval['href'] is not None:
                    prop_map[MetaNames.TRAINING_LIB_UID] = self.model_entity.get('training_lib')['href']
            if self.model_entity.get('space') is not None:
                runtimeval = self.model_entity.get('space')
                if runtimeval['href'] is not None:
                    prop_map[MetaNames.SPACE_UID] = self.model_entity.get('space')['href']

            if self.model_entity.get('project') is not None:
                projectval = self.model_entity.get('project')
                if projectval['href'] is not None:
                    prop_map[MetaNames.PROJECT_UID] = self.model_entity.get('project')['href']

               # TODO Fix Evaluation
        #         try:
        #             evaluation_data = self.model_entity.get('evaluation', None)
        #             if evaluation_data is not None:
        #                 prop_map[MetaNames.EVALUATION_METHOD] = evaluation_data.get('method', None)
        #                 if evaluation_data.get('metrics', None) is not None:
        #                     prop_map[MetaNames.EVALUATION_METRICS] = evaluation_data.get('metrics')
        #         #    evaluation_data = self.model_entity['evaluation']
        #         #    if evaluation_data is not None:t
        #         #        prop_map[MetaNames.EVALUATION_METHOD] = evaluation_data['method']
        #         #        if evaluation_data['metrics'] is not None:
        #         #           metrics = evaluation_data['metrics']
        #         #            prop_map[MetaNames.EVALUATION_METRICS] = metrics
        #         except KeyError:
        #             print("No Evlauation method given")

        #name = ''.join(map(lambda x: chr(ord(x)), self.model_output.entity['name']))
            name = self.model_entity.get('name')

            model_id = self.model_metadata.guid

            model_artifact = model_artifact_builder(
                uid=model_id,
                name=name,
                meta_props=MetaProps(prop_map)
            )

            model_artifact.client = self.client

            if model_artifact.meta.prop(MetaNames.TYPE).startswith('mllib'):
                model_artifact._pipeline_url = self.model_entity.get('pipeline')['href']

            if model_artifact.meta.prop(MetaNames.TYPE).startswith('wml_1.'):
                model_artifact._pipeline_url = self.model_entity.get('pipeline')['href']

            if self.model_entity is not None:
                version_props = MetaProps({})
                # content_status, content_location = None, None
                if 'content_status' in self.model_entity:
                    if self.model_entity['content_status'] is not None:
                        version_props.add(MetaNames.CONTENT_STATUS, self.model_entity['content_status'])

                if 'import' in self.model_entity:
                    if self.model_entity['import'] is not None:
                        version_props.add(MetaNames.IMPORT, self.model_entity['import'])

                if 'hyper_parameters' in self.model_entity:
                    if self.model_entity['hyper_parameters'] is not None:
                        version_props.add(MetaNames.HYPER_PARAMETERS, self.model_entity['hyper_parameters'])
                        version_props.add(MetaNames.HYPER_PARAMETERS, self.model_entitya['hyper_parameters'])

                model_artifact.meta.merge(version_props)
                model_artifact._content_href = "/v4/models/"+self.model_metadata.guid+"/content"
            else:
                model_artifact._content_href = None

        return model_artifact



from .abstract_autoai_test import AbstractTestAutoAIAsync, AbstractTestAutoAISync
from .abstract_deployment_webservice import AbstractTestWebservice
from .abstract_deployment_batch import AbstractTestBatch
from .abstract_test_iris_wml_autoai_multiclass_connections import AbstractTestAutoAIRemote
from .abstract_test_iris_using_database_connection import AbstractTestAutoAIDatabaseConnection
from .abstract_test_iris_using_database_data_asset import AbstractTestAutoAIConnectedAsset
from .abstract_obm_autoai_test import AbstractTestOBM
from .abstract_timeseries_test import AbstractTestTSAsync



import unittest
import datetime
from ibm_watson_machine_learning.tests.ICP.preparation_and_cleaning import *
from ibm_watson_machine_learning.tests.ICP.models_preparation import *
import logging
from ibm_watson_machine_learning.runtimes import LibraryDefinition
from ibm_watson_machine_learning.utils import SCIKIT_LEARN_FRAMEWORK


class TestRuntimeSpec(unittest.TestCase):
    runtime_uid = None
    runtime_url = None
    model_uid = None
    library_uid = None
    space_id = None
    logger = logging.getLogger(__name__)

    @classmethod
    def setUpClass(self):
        TestRuntimeSpec.logger.info("Service Instance: setting up credentials")

        self.wml_credentials = get_wml_credentials()
        self.client = get_client()
        self.model_path = os.path.join(os.getcwd(), 'artifacts', 'scikit_xgboost_model.tar.gz')
        self.custom_library_path = os.path.join(os.getcwd(), 'artifacts', 'scikit_xgboost_model.tar.gz') # TODO

        self.space = self.client.spaces.store({self.client.spaces.ConfigurationMetaNames.NAME: "test_case_TestRuntimeSpec"})
        self.space_id = self.client.spaces.get_uid(self.space)
        self.client.set.default_space(self.space_id)

    # def test_1_service_instance_details(self):
    #     TestRuntimeSpec.logger.info("Check client ...")
    #     self.assertTrue(self.client.__class__.__name__ == 'APIClient')
    #
    #     TestRuntimeSpec.logger.info("Getting instance details ...")
    #     details = self.client.service_instance.get_details()
    #
    #     TestRuntimeSpec.logger.debug(details)
    #     self.assertTrue("published_models" in str(details))
    #     self.assertEqual(type(details), dict)

    def test_1_create_runtime(self):
        lib_meta = {
            self.client.runtimes.LibraryMetaNames.NAME: "libraries_custom1Check_CP4D_test_build",
            self.client.runtimes.LibraryMetaNames.DESCRIPTION: "custom libraries for scoring",
            self.client.runtimes.LibraryMetaNames.FILEPATH: self.custom_library_path,
            self.client.runtimes.LibraryMetaNames.VERSION: "1.0",
            self.client.runtimes.LibraryMetaNames.PLATFORM: {"name": "python", "versions": ["3.6"]}
        }
        custom_library_details = self.client.runtimes.store_library(lib_meta)
        TestRuntimeSpec.custom_library_uid = self.client.runtimes.get_library_uid(custom_library_details)

        meta = {
            self.client.runtimes.ConfigurationMetaNames.NAME: "runtime_spec_python_CP4D1_test_build",
            self.client.runtimes.ConfigurationMetaNames.DESCRIPTION: "test",
            self.client.runtimes.ConfigurationMetaNames.PLATFORM: {
                "name": "python",
                "version": "3.5"
            },
            self.client.runtimes.ConfigurationMetaNames.LIBRARIES_UIDS: [TestRuntimeSpec.custom_library_uid]
        }
        runtime_details = self.client.runtimes.store(meta)
        TestRuntimeSpec.runtime_uid = self.client.runtimes.get_uid(runtime_details)
        TestRuntimeSpec.runtime_url = self.client.runtimes.get_href(runtime_details)

        self.assertTrue(TestRuntimeSpec.runtime_uid is not None)

    def test_2_download_library_content(self):
        try:
            os.remove('library.tar.gz')
        except:
            pass
        self.client.repository.download(TestRuntimeSpec.custom_library_uid, filename='library.tar.gz')
        try:
            os.remove('library.tar.gz')
        except:
            pass

    def test_3_store_model(self):
        self.logger.info("Publishing scikit-learn model ...")

        self.client.repository.ModelMetaNames.show()

        model_props = {self.client.repository.ModelMetaNames.RUNTIME_UID: TestRuntimeSpec.runtime_uid,
                       self.client.repository.ModelMetaNames.NAME: "LOCALLY created Digits prediction model",
                       self.client.repository.ModelMetaNames.TYPE: "scikit-learn_0.20"
                       }
        published_model_details = self.client.repository.store_model(model=self.model_path, meta_props=model_props)
        TestRuntimeSpec.model_uid = self.client.repository.get_model_uid(published_model_details)
        TestRuntimeSpec.model_url = self.client.repository.get_model_href(published_model_details)
        self.logger.info("Published model ID:" + str(TestRuntimeSpec.model_uid))
        self.logger.info("Published model URL:" + str(TestRuntimeSpec.model_url))
        self.assertIsNotNone(TestRuntimeSpec.model_uid)
        self.assertIsNotNone(TestRuntimeSpec.model_url)
    def test_4_get_details(self):
        details = self.client.repository.get_details(TestRuntimeSpec.model_uid)
        runtime_url = self.client.runtimes.get_href(details)
        self.assertTrue(runtime_url == TestRuntimeSpec.runtime_url)

    def test_5_delete_model(self):
        self.client.repository.delete(TestRuntimeSpec.model_uid)

    def test_6_delete_runtime(self):

        self.client.runtimes.delete(TestRuntimeSpec.runtime_uid)
        self.client.runtimes.delete_library(TestRuntimeSpec.custom_library_uid)
        self.client.spaces.delete(TestRuntimeSpec.space_id)


if __name__ == '__main__':
    unittest.main()



import unittest

import ibm_boto3
from ibm_watson_machine_learning.experiment import AutoAI
from ibm_watson_machine_learning.helpers.connections import DataConnection, S3Location
from ibm_watson_machine_learning.utils.autoai.errors import WMLClientError
from ibm_watson_machine_learning.tests.utils import bucket_exists, create_bucket, is_cp4d, create_connection_to_cos
from ibm_watson_machine_learning.tests.autoai.abstract_tests_classes import AbstractTestAutoAISync,\
    AbstractTestWebservice

from ibm_watson_machine_learning.utils.autoai.enums import PredictionType, Metrics


class TestAutoAIRemote(AbstractTestAutoAISync, AbstractTestWebservice, unittest.TestCase):
    """
    The test can be run on CPD
    """

    cos_resource = None
    # TODO: Chancge to xlsx for cpd 4.0.3
    # data_location = './autoai/data/xlsx/CarPrice_bank__two_sheets.xlsx'
    # sheet_name = 'bank'
    # sheet_number = 1

    # data_cos_path = 'data/CarPrice_bank__two_sheets.xlsx'

    data_location = './autoai/data/bank.csv'
    data_cos_path = "bank.csv"

    SPACE_ONLY = True

    OPTIMIZER_NAME = "Bank test sdk"

    target_space_id = None

    experiment_info = dict(
        name=OPTIMIZER_NAME,
        desc='test description',
        prediction_type=PredictionType.BINARY,
        prediction_column='y',
        positive_label='yes',
        scoring=Metrics.ACCURACY_SCORE,
        holdout_size=0.1,
        max_number_of_estimators=1,
        # excel_sheet=sheet_name
    )

    def test_00b_prepare_COS_instance_and_connection(self):
        TestAutoAIRemote.connection_id, TestAutoAIRemote.bucket_name = create_connection_to_cos(
            wml_client=self.wml_client,
            cos_credentials=self.cos_credentials,
            cos_endpoint=self.cos_endpoint,
            bucket_name=self.bucket_name,
            save_data=True,
            data_path=self.data_location,
            data_cos_path=self.data_cos_path)

        print("Connection id: ", self.connection_id)

        self.assertIsInstance(self.connection_id, str)

    def test_00c_prepare_connected_data_asset(self):
        asset_details = self.wml_client.data_assets.store({
            self.wml_client.data_assets.ConfigurationMetaNames.CONNECTION_ID: self.connection_id,
            self.wml_client.data_assets.ConfigurationMetaNames.NAME: "Bank - training asset",
            self.wml_client.data_assets.ConfigurationMetaNames.DATA_CONTENT_NAME: f"{self.bucket_name}/{self.data_cos_path}"
        })
        print(asset_details)

        TestAutoAIRemote.asset_id = self.wml_client.data_assets.get_id(asset_details)
        self.assertIsInstance(self.asset_id, str)

    def test_02_data_reference_setup(self):
        TestAutoAIRemote.data_connection = DataConnection(data_asset_id=self.asset_id)
        TestAutoAIRemote.results_connection = None

        self.assertIsNotNone(obj=TestAutoAIRemote.data_connection)
        self.assertIsNone(obj=TestAutoAIRemote.results_connection)

    # def test_99_delete_connection_and_connected_data_asset(self):
    #     self.wml_client.data_assets.delete(self.asset_id)
    #     self.wml_client.connections.delete(self.connection_id)
    #
    #     with self.assertRaises(WMLClientError):
    #         self.wml_client.data_assets.get_details(self.asset_id)
    #         self.wml_client.connections.get_details(self.connection_id)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from ibm_watson_machine_learning.tests.base.abstract.abstract_deep_learning_test import \
    AbstractDeepLearningExperimentTest


class TestPyTorchTraining(AbstractDeepLearningExperimentTest, unittest.TestCase):
    """
    Test case checking the scenario of training an PyTorch model
    using experiment.
    """

    model_definition_name = "pytorch_model_definition"
    training_name = "test_pytorch_training"
    training_description = "torch-test-experiment"
    experiment_name = "test_torch_experiment"
    execution_command = "python torch_mnist.py --epochs 1"

    software_specification_name = "pytorch-onnx_1.7-py3.8"
    data_location = os.path.join(os.getcwd(), "base", "datasets", "tensorflow", "mnist.npz")
    data_cos_path = "mnist.npz"
    model_paths = [
        os.path.join(os.getcwd(), "base", "artifacts", "pytorch", "pytorch-model.zip")
    ]



import unittest
from pprint import pprint

import pandas as pd

from ibm_watson_machine_learning.experiment import AutoAI
from ibm_watson_machine_learning.workspace import WorkSpace
from ibm_watson_machine_learning.deployment import WebService
from ibm_watson_machine_learning.experiment.autoai.runs import AutoPipelinesRuns
from ibm_watson_machine_learning.helpers import DataConnection
from ibm_watson_machine_learning.experiment.autoai.optimizers import RemoteAutoPipelines
from ibm_watson_machine_learning.tests.utils import get_wml_credentials
from sklearn.pipeline import Pipeline


class TestAutoPipelinesRuns(unittest.TestCase):
    experiment: 'AutoAI' = None
    optimizer_runs: 'AutoPipelinesRuns' = None
    optimizer: 'RemoteAutoPipelines' = None
    pipeline: 'Pipeline' = None
    deployed_pipeline = None
    ws: 'WorkSpace' = None
    service: 'WebService' = None

    space_id = None
    project_id = None

    run_id = '08754ca1-e0fd-4d69-9319-eb3dbbc28757'
    data_location = './autoai/data/bank.csv'
    data = None
    X = None
    y = None

    @classmethod
    def setUp(cls) -> None:
        """
        Load WML credentials from config.ini file based on ENV variable.
        """
        cls.data = pd.read_csv(cls.data_location)
        cls.X = cls.data.drop(['y'], axis=1)
        cls.y = cls.data['y']
        cls.wml_credentials = get_wml_credentials()

        cls.ws = WorkSpace(wml_credentials=cls.wml_credentials,
                           project_id=cls.project_id,
                           space_id=cls.space_id)

    def test_01__initialize_AutoAI_experiment__pass_credentials__object_initialized(self):
        TestAutoPipelinesRuns.experiment = AutoAI(self.ws)

        self.assertIsInstance(self.experiment, AutoAI, msg="Experiment is not of type AutoAI.")

    def test_02__check_AutoPipelinesRuns__historical_runs_in_AutoAI__runs_initialized(self):
        self.assertIsInstance(self.experiment.runs, AutoPipelinesRuns,
                              msg="Experiment not initialized AutoPipelinesRuns (historical runs) correctly.")

    def test_03__runs_list__list_runs__runs_listed(self):
        runs = self.experiment.runs.list()
        print(runs)

        self.assertIsInstance(runs, pd.DataFrame, msg="Listed runs are not in the format of pandas DataFrame")

    def test_04__runs_list__list_filtered_runs__runs_listed(self):
        runs = self.experiment.runs(filter="test name").list()
        print(runs)

        self.assertIsInstance(runs, pd.DataFrame, msg="Listed runs are not in the format of pandas DataFrame")

    def test_05__get_params__get_configuration_parameters_of_remote_auto_pipeline__parameters_listed(self):
        parameters = self.experiment.runs.get_params()
        print(parameters)
        self.assertIsInstance(parameters, dict, msg='Config parameters are not a dictionary instance.')

    def test_06__get_run_details__fetching_run_parameters__run_parameters_fetched(self):
        parameters = self.experiment.runs.get_run_details()
        pprint(parameters)

    def test_07__get_run_details__fetching_run_parameters_for_run_id__run_parameters_fetched(self):
        parameters = self.experiment.runs.get_run_details(run_id=self.run_id)
        pprint(parameters)

    def test_08__get_data_connections__return_list_of_data_connection_objects_from_run(self):
        data_connections = self.experiment.runs.get_data_connections(run_id=self.run_id)
        self.assertIsInstance(data_connections, list, msg="There should be a list container returned")
        self.assertIsInstance(data_connections[0], DataConnection,
                              msg="There should be a DataConnection object returned")

    def test_09__get_optimizer__get_RemoteAutoPipelines_object__new_object_returned(self):
        TestAutoPipelinesRuns.optimizer = self.experiment.runs.get_optimizer(run_id=self.run_id)

        print(type(self.optimizer))
        self.assertIsInstance(self.optimizer, RemoteAutoPipelines)

    def test_10__run_status__get_status_of_a_run__status_fetched(self):
        status = self.optimizer.get_run_status()
        print(status)

    def test_11__results__listing_all_pipelines_from_wml__all_pipelines_listed(self):
        pipelines_details = self.optimizer.summary()
        print(pipelines_details)

    def test_12__get_pipeline_params__fetch_specific_pipeline_parameters__parameters_fetched_as_dict(self):
        pipeline_params = self.optimizer.get_pipeline_details(pipeline_name='Pipeline_1')
        print(pipeline_params)

    def test_13__get_pipeline_params__fetch_best_pipeline_parameters__parameters_fetched_as_dict(self):
        best_pipeline_params = self.optimizer.get_pipeline_details()
        print(best_pipeline_params)

    def test_14__get_pipeline__load_specified_pipeline__pipeline_loaded(self):
        TestAutoPipelinesRuns.pipeline = self.optimizer.get_pipeline(pipeline_name='Pipeline_4',
                                                                     astype=AutoAI.PipelineTypes.SKLEARN)
        print(f"Fetched pipeline type: {type(self.pipeline)}")

        self.assertIsInstance(self.pipeline, Pipeline, msg="Fetched pipeline is not of sklearn type.")

    def test_15__get_pipeline_and_persist_it(self):
        try:
            from sklearn.externals import joblib
        except ImportError:
            import joblib

        pipeline = self.optimizer.get_pipeline(pipeline_name="Pipeline_1",
                                               astype=AutoAI.PipelineTypes.SKLEARN,
                                               persist=True)

        exception = None
        try:
            model = joblib.load('./Pipeline_1.pickle')

        except Exception as exception:
            self.assertIsNone(exception, msg=f"Cannot load saved pipeline, Error: {exception}")

    def test_16__deploy__deploy_best_computed_pipeline_from_autoai_on_wml(self):
        TestAutoPipelinesRuns.service = WebService(self.ws)

        self.service.create(
            experiment_run_id=self.run_id,
            model=self.pipeline,
            deployment_name='autoAI deployment')

    def test_17__predict__score_deployment(self):
        results = self.service.score(payload=self.X.iloc[:10])
        print(results)


if __name__ == '__main__':
    unittest.main()



# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = [
    "discover_input_data",
    "get_data_asset_attachment",
    "prepare_cos_data_location",
    "prepare_interaction_props_for_cos",
    "prepare_payload_for_excel",
    "check_location",
    "try_import_pyarrow"
]

import json
import requests
from ibm_boto3 import client
from ibm_botocore.client import Config
from typing import Tuple, TYPE_CHECKING
import pkg_resources
from packaging import version
from warnings import warn
from subprocess import check_call
from sys import executable

from .errors import UnsupportedConnection, UnsupportedOutputConnection, WrongDatabaseSchemaOrTable, MissingFileName
from ibm_watson_machine_learning.utils.autoai.errors import CannotInstallLibrary

if TYPE_CHECKING:
    from ibm_watson_machine_learning import APIClient

BOTO_CONFIG_DOWNLOAD = Config(connect_timeout=60, read_timeout=60, retries={'total_max_attempts': 5})


def discover_input_data(data_location):
    cos = client(
        service_name='s3',
        aws_access_key_id=data_location["connection"]["access_key_id"],
        aws_secret_access_key=data_location["connection"]["secret_access_key"],
        endpoint_url=data_location["connection"]["endpoint_url"],
        verify=False,
        config=BOTO_CONFIG_DOWNLOAD
    )

    parent_folder = str(data_location["location"].get("path", '')).replace("./", "", 1)
    try:
        csv_keys = cos.list_objects_v2(
            Bucket=data_location["location"]["bucket"],
            FetchOwner=False,
            Prefix=parent_folder + "/features/part-")

        if 'Contents' in csv_keys:
            parts = []
            for obj in csv_keys['Contents']:
                parts.append(obj['Key'])
            return parts
    except Exception as e:
        pass

    return [data_location["location"]["path"]]


def get_data_asset_attachment(data_location: dict, wml_client: 'APIClient') -> dict:
    """Here we can download data asset attachment for further inspection."""
    # note: try to support new design with id in connection part, if cannot, try to use old one with location

    if 'href' in data_location:
        data_location['id'] = data_location['href'].split('/')[3].split('?')[0]

    data_asset = wml_client.data_assets.get_details(data_location['id'])

    try:
        attachment_id = str(data_asset['attachments'][0]['id'])

    except KeyError:
        attachment_id = data_asset['metadata']['attachment_id']

    attachment_url = f"{wml_client.service_instance._href_definitions.get_attachments_href(data_location['id'])}/{attachment_id}"

    if wml_client.ICP:
        attachment = requests.get(attachment_url, params=wml_client._params(), headers=wml_client._get_headers(),
                                  verify=False)

    else:
        attachment = requests.get(attachment_url, params=wml_client._params(), headers=wml_client._get_headers())

    attachment_content = attachment.content
    attachment_json = json.loads(attachment_content)

    return attachment_json


def prepare_cos_data_location(data_location, wml_client: 'APIClient', out=False):
    """To be able to interact with COS directly, we need to prepare credentials for COS client.
    This is needed for COS introspection eg. for OBM data parts.
    """
    attachment_json = {}

    if data_location['type'] == 'data_asset':
        attachment_json = get_data_asset_attachment(data_location, wml_client)
        connection_details = wml_client.connections.get_details(attachment_json['connection_id'])

    elif data_location['type'] == 'connection_asset':
        connection_details = wml_client.connections.get_details(data_location['connection']['id'])

    else:
        raise UnsupportedConnection(conn_type=data_location['type'])

    # Note: try to find out if we have connection_asset pointing out to S3 COS
    if 'secret_key' in connection_details['entity']['properties']:
        bucket = connection_details['entity']['properties'].get('bucket')
        path = data_location['location'].get('path')

        # training service did not specified which version is correct for now
        if path is None:
            path = data_location['location'].get('file_name')

        new_data_location = {
            "connection": {
                "access_key_id": connection_details['entity']['properties']['access_key'],
                "secret_access_key": connection_details['entity']['properties']['secret_key'],
                "endpoint_url": connection_details['entity']['properties']['url'],
            },
            "location": {
                "bucket": bucket if bucket is not None else data_location['location'].get('bucket'),
                "path": path if path is not None else attachment_json.get('connection_path')
            }
        }

    elif 'cos_hmac_keys' in connection_details['entity']['properties'].get('credentials', ''):
        creds = json.loads(connection_details['entity']['properties']['credentials'])

        bucket = connection_details['entity']['properties'].get('bucket')
        path = data_location['location'].get('path')

        # training service did not specified which version is correct for now
        if path is None:
            path = data_location['location'].get('file_name')

        new_data_location = {
            "connection": {
                "access_key_id": creds['cos_hmac_keys']['access_key_id'],
                "secret_access_key": creds['cos_hmac_keys']['secret_access_key'],
                "endpoint_url": connection_details['entity']['properties']['url'],
            },
            "location": {
                "bucket": bucket if bucket is not None else data_location['location'].get('bucket'),
                "path": path if path is not None else attachment_json.get('connection_path')
            }
        }

    else:
        if out:
            raise UnsupportedOutputConnection(
                connection_id=attachment_json['connection_id'] if 'connection_id' in attachment_json else
                data_location['connection']['id'],
                reason="Supported output types are: COS for cloud and File System for CP4D. "
                       "Make sure that connection has correct COS credentials specified.")

        else:
            raise UnsupportedConnection(
                conn_type=data_location['type'],
                reason="Make sure that connection has correct COS credentials specified.")

    return new_data_location


def prepare_interaction_props_for_cos(params: dict, input_key_file) -> dict:
    """If user specified properties for training dataset as sheet_name, delimiter etc. we need to
    pass them as interaction properties for Flight Service.
    """
    file_format = None
    interaction_properties = {}

    encoding = params.get('encoding')
    if ".xls" in input_key_file or ".xlsx" in input_key_file:
        file_format = "excel"
        interaction_properties["sheet_name"] = str(params.get('excel_sheet', 0))
    elif '.csv' in input_key_file:
        if encoding is not None:
            interaction_properties["encoding"] = encoding
        input_file_separator = params.get('input_file_separator', ',')
        if input_file_separator != ",":
            file_format = "delimited"
            interaction_properties["field_delimiter"] = input_file_separator
        else:
            file_format = "csv"

    elif '.parquet' in input_key_file or '.prq' in input_key_file:
        file_format = 'parquet'

    if file_format is not None:
        interaction_properties["file_format"] = file_format

    return interaction_properties


def prepare_payload_for_excel(cos_data_location: dict, cos_interaction_properties: dict,
                              params: dict, input_key_file: str) -> Tuple[dict, str]:
    """If we have to download excel file we need to specify sheet_name. In some cases when user specifies
    input data as data_asset, the path is appended with sheet_name already so we need to remove it and place
    into interaction properties.
    """
    # data asset set path with spread_sheet name, we need to remove it
    if cos_data_location['location']['path'] is not None:
        if '.xlsx' in cos_data_location['location']['path']:
            if (not cos_data_location['location']['path'].endswith('.xlsx') and
                    'excel_sheet' not in str(params)):
                cos_interaction_properties['sheet_name'] = (
                    cos_data_location['location']['path'].split('/')[-1])

            input_key_file = cos_data_location['location']['path'].split('.xlsx')[0] + '.xlsx'

        elif '.xls' in cos_data_location['location']['path']:
            if (not cos_data_location['location']['path'].endswith('.xls') and
                    'excel_sheet' not in str(params)):
                cos_interaction_properties['sheet_name'] = (
                    cos_data_location['location']['path'].split('/')[-1])

            input_key_file = cos_data_location['location']['path'].split('.xls')[0] + '.xls'

    return cos_interaction_properties, input_key_file


def check_location(data_location: dict, connection_type: str) -> None:
    if data_location['type'] == 'connection_asset':
        if connection_type == 'database':
            if 'schema_name' not in data_location['location'] and 'table_name' not in data_location['location']:
                raise WrongDatabaseSchemaOrTable(reason="Connection asset requires to have specified schema_name "
                                                        "and table_name for database connection.")

        elif connection_type == 'file':
            if 'file_name' not in data_location['location']:
                raise MissingFileName(reason="Connection asset requires to have specified a 'file_name'.")


def try_import_pyarrow():
    """
    Check if pyarrow package is installed in local environment, if not, just download and install it.
    """
    pyarrow_version = '3.0.0'
    try:
        installed_module_version = pkg_resources.get_distribution('pyarrow').version

        if version.parse(installed_module_version) < version.parse(pyarrow_version):
            warn(f"\"pyarrow\" package version is lower than {pyarrow_version}."
                 f"If you want to download data from Database Connection, you need to have pyarrow installed. "
                 f"If you are working on conda environment, you can try to install "
                 f"this package from conda-forge repository."
                 f"Installing version {pyarrow_version}")

            try:
                check_call([executable, "-m", "pip", "install", f"pyarrow=={pyarrow_version}"])

            except Exception as e:
                raise CannotInstallLibrary(value_name=e,
                                           reason="pyarrow failed to install. Please install it manually.")

    except pkg_resources.DistributionNotFound as e:
        warn(f"\"pyarrow\" is not installed. If you want to download data from Database Connection, "
             f"you need to have pyarrow installed. If you are working on conda environment, you can try to install "
             f"this package from conda-forge repository."
             f"Installing version {pyarrow_version}")

        try:
            check_call([executable, "-m", "pip", "install", f"pyarrow=={pyarrow_version}"])

        except Exception as e:
            raise CannotInstallLibrary(value_name=e,
                                       reason="pyarrow failed to install. Please install it manually.")



import unittest
import time
import pandas as pd
import logging
from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure
from ibm_watson_machine_learning.tests.ICP.preparation_and_cleaning import *


class TestWMLClientWithDO(unittest.TestCase):
    deployment_uid = None
    model_uid = None
    scoring_url = None
    scoring_uid = None
    logger = logging.getLogger(__name__)

    @classmethod
    def setUpClass(self):
        TestWMLClientWithDO.logger.info("Service Instance: setting up credentials")

        self.wml_credentials = get_wml_credentials()
        self.client = get_client()

        self.cos_credentials = get_cos_credentials()
        self.cos_resource_crn = self.cos_credentials['resource_instance_id']

        self.space_name = str(uuid.uuid4())

        metadata = {
            self.client.spaces.ConfigurationMetaNames.NAME: 'space' + self.space_name,
            self.client.spaces.ConfigurationMetaNames.DESCRIPTION: self.space_name + ' description',
            self.client.spaces.ConfigurationMetaNames.STORAGE: {
                "type": "bmcos_object_storage",
                "resource_crn": self.cos_resource_crn
            }
        }

        self.space = self.client.spaces.store(meta_props=metadata, background_mode=False)

        TestWMLClientWithDO.space_id = self.client.spaces.get_id(self.space)
        print("space_id: ", TestWMLClientWithDO.space_id)
        self.client.set.default_space(TestWMLClientWithDO.space_id)
       # self.model_path = os.path.join(os.getcwd(), 'artifacts', 'customer-satisfaction-prediction.str')

        # TestWMLClientWithDO.logger.info("Service Instance: setting up credentials")
        # self.wml_credentials = get_wml_credentials()
        # self.client = get_client()
        self.model_path = os.path.join(os.getcwd(), 'artifacts', 'do-model.tar.gz')

    # def test_01_set_space(self):
    #     space = self.client.spaces.store({self.client.spaces.ConfigurationMetaNames.NAME: "DO_test_case"})
    #
    #     TestWMLClientWithDO.space_id = self.client.spaces.get_uid(space)
    #     self.client.set.default_space(TestWMLClientWithDO.space_id)
    #     self.assertTrue("SUCCESS" in self.client.set.default_space(TestWMLClientWithDO.space_id))


    def test_02_publish_do_model_in_repository(self):
        TestWMLClientWithDO.logger.info("Saving trained model in repo ...")
        TestWMLClientWithDO.logger.debug("Model path: {}".format(self.model_path))

        self.client.repository.ModelMetaNames.show()

        sw_spec_uid = self.client.software_specifications.get_uid_by_name("do_12.9")
        output_data_schema = [{'id': 'stest',
                              'type': 'list',
                              'fields': [{'name': 'age', 'type': 'float'},
                                         {'name': 'sex', 'type': 'float'},
                                         {'name': 'cp', 'type': 'float'},
                                         {'name': 'restbp', 'type': 'float'},
                                         {'name': 'chol', 'type': 'float'},
                                         {'name': 'fbs', 'type': 'float'},
                                         {'name': 'restecg', 'type': 'float'},
                                         {'name': 'thalach', 'type': 'float'},
                                         {'name': 'exang', 'type': 'float'},
                                         {'name': 'oldpeak', 'type': 'float'},
                                         {'name': 'slope', 'type': 'float'},
                                         {'name': 'ca', 'type': 'float'},
                                         {'name': 'thal', 'type': 'float'}]
                               }, {'id': 'teste2',
                 'type': 'test',
                 'fields': [{'name': 'age', 'type': 'float'},
                            {'name': 'sex', 'type': 'float'},
                            {'name': 'cp', 'type': 'float'},
                            {'name': 'restbp', 'type': 'float'},
                            {'name': 'chol', 'type': 'float'},
                            {'name': 'fbs', 'type': 'float'},
                            {'name': 'restecg', 'type': 'float'},
                            {'name': 'thalach', 'type': 'float'},
                            {'name': 'exang', 'type': 'float'},
                            {'name': 'oldpeak', 'type': 'float'},
                            {'name': 'slope', 'type': 'float'},
                            {'name': 'ca', 'type': 'float'},
                            {'name': 'thal', 'type': 'float'}]}]
        model_meta_props = {self.client.repository.ModelMetaNames.NAME: "LOCALLY created DO model",
                       self.client.repository.ModelMetaNames.TYPE: "do-docplex_12.9",
                       self.client.repository.ModelMetaNames.SOFTWARE_SPEC_UID: sw_spec_uid,
                       self.client.repository.ModelMetaNames.OUTPUT_DATA_SCHEMA: output_data_schema
                            }
        published_model = self.client.repository.store_model(model=self.model_path, meta_props=model_meta_props)
        TestWMLClientWithDO.model_uid = self.client.repository.get_model_uid(published_model)
        TestWMLClientWithDO.logger.info("Published model ID:" + str(TestWMLClientWithDO.model_uid))
        self.assertIsNotNone(TestWMLClientWithDO.model_uid)


    def test_04_get_details(self):
        TestWMLClientWithDO.logger.info("Get details")
        self.assertIsNotNone(self.client.repository.get_details())
        det = self.client.repository.get_details()
        print(det)

    def test_05_get_model_details(self):
        TestWMLClientWithDO.logger.info("Get model details")
        details = self.client.repository.get_details(TestWMLClientWithDO.model_uid)
        import json
        print(details)
        self.assertIsNotNone(self.client.repository.get_model_details(TestWMLClientWithDO.model_uid))

    def test_06_create_revision(self):
        TestWMLClientWithDO.logger.info("Create Revision")
        self.client.repository.create_model_revision(TestWMLClientWithDO.model_uid)
        self.client.repository.list_models_revisions(TestWMLClientWithDO.model_uid)
        model_meta_props = {self.client.repository.ModelMetaNames.NAME: "updated do model",
                            }
        published_model_updated = self.client.repository.update_model(TestWMLClientWithDO.model_uid,
                                                                      model_meta_props, self.model_path)
        self.assertIsNotNone(TestWMLClientWithDO.model_uid)
        self.assertTrue("updated do model" in str(published_model_updated))
        self.client.repository.create_model_revision(TestWMLClientWithDO.model_uid)
        rev_details = self.client.repository.get_model_revision_details(TestWMLClientWithDO.model_uid, 2)
        self.assertTrue("updated do model" in str(rev_details))

    # def test_07_create_deployment(self):
    #     TestWMLClientWithDO.logger.info("Create deployment")
    #
    #     deploy_meta = {
    #         self.client.deployments.ConfigurationMetaNames.NAME: "deployment_DO",
    #         self.client.deployments.ConfigurationMetaNames.DESCRIPTION: "DO deployment",
    #         self.client.deployments.ConfigurationMetaNames.BATCH: {},
    #         self.client.deployments.ConfigurationMetaNames.HARDWARE_SPEC: {"name": "S", "num_nodes": 1}
    #     }
    #
    #
    #     deployment_details = self.client.deployments.create(TestWMLClientWithDO.model_uid,
    #                                                         meta_props=deploy_meta)
    #
    #     TestWMLClientWithDO.logger.debug("Deployment details: {}".format(deployment_details))
    #
    #     TestWMLClientWithDO.deployment_uid = self.client.deployments.get_uid(deployment_details)
    #     TestWMLClientWithDO.logger.debug("Deployment uid: {}".format(TestWMLClientWithDO.deployment_uid))
    #
    #
    # def test_08_get_deployment_details(self):
    #     TestWMLClientWithDO.logger.info("Get deployment details")
    #     deployment_details = self.client.deployments.get_details(deployment_uid=TestWMLClientWithDO.deployment_uid)
    #     self.assertIsNotNone(deployment_details)
    #
    #
    # def test_09_create_job(self):
    #     TestWMLClientWithDO.logger.info("Create job details")
    #     TestWMLClientWithDO.logger.debug("Create job")
    #
    #     # initialize list of lists
    #     diet_food = pd.DataFrame([["Roasted Chicken", 0.84, 0, 10],
    #                               ["Spaghetti W/ Sauce", 0.78, 0, 10],
    #                               ["Tomato,Red,Ripe,Raw", 0.27, 0, 10],
    #                               ["Apple,Raw,W/Skin", 0.24, 0, 10],
    #                               ["Grapes", 0.32, 0, 10],
    #                               ["Chocolate Chip Cookies", 0.03, 0, 10],
    #                               ["Lowfat Milk", 0.23, 0, 10],
    #                               ["Raisin Brn", 0.34, 0, 10],
    #                               ["Hotdog", 0.31, 0, 10]], columns=["name", "unit_cost", "qmin", "qmax"])
    #
    #     diet_food_nutrients = pd.DataFrame([
    #         ["Spaghetti W/ Sauce", 358.2, 80.2, 2.3, 3055.2, 11.6, 58.3, 8.2],
    #         ["Roasted Chicken", 277.4, 21.9, 1.8, 77.4, 0, 0, 42.2],
    #         ["Tomato,Red,Ripe,Raw", 25.8, 6.2, 0.6, 766.3, 1.4, 5.7, 1],
    #         ["Apple,Raw,W/Skin", 81.4, 9.7, 0.2, 73.1, 3.7, 21, 0.3],
    #         ["Grapes", 15.1, 3.4, 0.1, 24, 0.2, 4.1, 0.2],
    #         ["Chocolate Chip Cookies", 78.1, 6.2, 0.4, 101.8, 0, 9.3, 0.9],
    #         ["Lowfat Milk", 121.2, 296.7, 0.1, 500.2, 0, 11.7, 8.1],
    #         ["Raisin Brn", 115.1, 12.9, 16.8, 1250.2, 4, 27.9, 4],
    #         ["Hotdog", 242.1, 23.5, 2.3, 0, 0, 18, 10.4]
    #     ], columns=["Food", "Calories", "Calcium", "Iron", "Vit_A", "Dietary_Fiber", "Carbohydrates", "Protein"])
    #
    #     diet_nutrients = pd.DataFrame([
    #         ["Calories", 2000, 2500],
    #         ["Calcium", 800, 1600],
    #         ["Iron", 10, 30],
    #         ["Vit_A", 5000, 50000],
    #         ["Dietary_Fiber", 25, 100],
    #         ["Carbohydrates", 0, 300],
    #         ["Protein", 50, 100]
    #     ], columns=["name", "qmin", "qmax"])
    #
    #     job_payload_ref = {
    #         self.client.deployments.DecisionOptimizationMetaNames.INPUT_DATA: [
    #             {
    #                 "id": "diet_food.csv",
    #                 "values": diet_food
    #             },
    #             {
    #                 "id": "diet_food_nutrients.csv",
    #                 "values": diet_food_nutrients
    #             },
    #             {
    #                 "id": "diet_nutrients.csv",
    #                 "values": diet_nutrients
    #             }
    #         ],
    #         self.client.deployments.DecisionOptimizationMetaNames.OUTPUT_DATA: [
    #             {
    #                 "id": ".*.csv"
    #             }
    #         ]
    #     }
    #
    #     TestWMLClientWithDO.job_details = self.client.deployments.create_job(TestWMLClientWithDO.deployment_uid,
    #                                                                          meta_props=job_payload_ref)
    #     TestWMLClientWithDO.job_id = self.client.deployments.get_job_uid(TestWMLClientWithDO.job_details)
    #
    #
    # def test_10_get_job_status(self):
    #     start_time = time.time()
    #     diff_time = start_time - start_time
    #     while True and diff_time < 10 * 60:
    #         time.sleep(3)
    #         response = self.client.deployments.get_job_status(TestWMLClientWithDO.job_id)
    #         if response['state'] == 'completed' or response['state'] == 'error' or response['state'] == 'failed':
    #             break
    #         diff_time = time.time() - start_time
    #
    #     self.assertIsNotNone(response)
    #     self.assertTrue(response['state'] == 'completed')
    #
    # def test_11_extract_and_display_solution(self):
    #     job_details_do = self.client.deployments.get_job_details(TestWMLClientWithDO.job_id)
    #     kpi = job_details_do['entity']['decision_optimization']['solve_state']['details']['KPI.Total Calories']
    #     print(kpi)
    #     TestWMLClientWithDO.logger.debug("The value of kpi is ", kpi)
    #     self.assertTrue(kpi == '2000.0')
    #
    #
    # def test_12_list_jobs(self):
    #     self.client.deployments.list_jobs()
    #
    # def test_12_delete_job(self):
    #     self.client.deployments.delete_job(TestWMLClientWithDO.job_id)
    #
    # def test_13_delete_deployment(self):
    #     TestWMLClientWithDO.logger.info("Delete deployment")
    #     self.client.deployments.delete(TestWMLClientWithDO.deployment_uid)

    def test_03_download_model(self):
        TestWMLClientWithDO.logger.info("Download model")
        try:
            os.remove('download_test_url2')
        except OSError:
            pass

        try:
            os.remove('download_test_url2')
        except IOError:
            pass

        self.client.repository.download(TestWMLClientWithDO.model_uid, filename='download_test_url2')
        try:
            os.remove('download_test_url2')
        except OSError:
            pass
       # self.assertRaises(WMLClientError, self.client.repository.download, TestWMLClientWithDO.model_uid, filename='download_test_uid')


    def test_14_delete_model(self):
        TestWMLClientWithDO.logger.info("Delete function")
        self.client.script.delete(TestWMLClientWithDO.model_uid)

    def test_15_delete_space(self):
        self.client.spaces.delete(TestWMLClientWithDO.space_id)


if __name__ == '__main__':
    unittest.main()



import unittest
import logging
from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure
from ibm_watson_machine_learning.tests.ICP.preparation_and_cleaning import *


class TestWMLClientWithHybrid(unittest.TestCase):
    deployment_uid = None
    model_uid = None
    scoring_url = None
    scoring_uid = None
    logger = logging.getLogger(__name__)

    @classmethod
    def setUpClass(self):
        TestWMLClientWithHybrid.logger.info("Service Instance: setting up credentials")
        self.wml_credentials = get_wml_credentials()
        self.client = get_client()
        self.model_path = os.path.join(os.getcwd(), 'artifacts', 'DrugSelectionAutoAI_model_content.gzip')

    def test_01_set_space(self):
        space = self.client.spaces.store({self.client.spaces.ConfigurationMetaNames.NAME: "test_case_autoaimodel"})

        TestWMLClientWithHybrid.space_id = self.client.spaces.get_uid(space)
        self.client.set.default_space(TestWMLClientWithHybrid.space_id)
        self.assertTrue("SUCCESS" in self.client.set.default_space(TestWMLClientWithHybrid.space_id))


    def test_02_publish_local_model_in_repository(self):
        TestWMLClientWithHybrid.logger.info("Saving trained model in repo ...")
        TestWMLClientWithHybrid.logger.debug("Model path: {}".format(self.model_path))

        self.client.repository.ModelMetaNames.show()

        sw_spec_uid = self.client.software_specifications.get_uid_by_name("hybrid_0.1")
        input_data_schema = [{
            "id": "auto_ai_kb_input_schema",
            "fields": [
                {
                  "name": "AGE",
                  "type": "int64"
                },
                {
                  "name": "SEX",
                  "type": "object"
                },
                {
                  "name": "BP",
                  "type": "object"
                },
                {
                  "name": "CHOLESTEROL",
                  "type": "object"
                },
                {
                  "name": "NA",
                  "type": "float64"
                },
                {
                  "name": "K",
                  "type": "float64"
                }
            ]
        }]

        trainind_data_ref = [
                {
                    "connection": {
                        "endpoint_url": "",
                        "access_key_id": "",
                        "secret_access_key": ""
                    },
                    "location": {
                        "bucket": "",
                        "path": ""
                    },
                    "type": "fs",
                    "schema": {
                        "id": "4cdb0a0a-1c69-43a0-a8c0-3918afc7d45f",
                        "fields": [
                            {
                                "metadata": {
                                    "name": "AGE",
                                    "scale": 0
                                },
                                "name": "AGE",
                                "nullable": True,
                                "type": "integer"
                            },
                            {
                                "metadata": {
                                    "name": "SEX",
                                    "scale": 0
                                },
                                "name": "SEX",
                                "nullable": True,
                                "type": "string"
                            },
                            {
                                "metadata": {
                                    "name": "BP",
                                    "scale": 0
                                },
                                "name": "BP",
                                "nullable": True,
                                "type": "string"
                            },
                            {
                                "metadata": {
                                    "name": "CHOLESTEROL",
                                    "scale": 0
                                },
                                "name": "CHOLESTEROL",
                                "nullable": True,
                                "type": "string"
                            },
                            {
                                "metadata": {
                                    "name": "NA",
                                    "scale": 6
                                },
                                "name": "NA",
                                "nullable": True,
                                "type": "decimal(12,6)"
                            },
                            {
                                "metadata": {
                                    "name": "K",
                                    "scale": 6
                                },
                                "name": "K",
                                "nullable": True,
                                "type": "decimal(13,6)"
                            }
                        ],
                        "type": "struct"
                    }
                }
        ]

        model_meta_props = {
                       self.client.repository.ModelMetaNames.NAME: "LOCALLY created DrugSelectionAutoAIModel",
                       self.client.repository.ModelMetaNames.TYPE: 'wml-hybrid_0.1',
                       self.client.repository.ModelMetaNames.SOFTWARE_SPEC_UID: sw_spec_uid,
                       self.client.repository.ModelMetaNames.INPUT_DATA_SCHEMA: input_data_schema,
                       self.client.repository.ModelMetaNames.TRAINING_DATA_REFERENCES: trainind_data_ref
                       }
        published_model = self.client.repository.store_model(model=self.model_path, meta_props=model_meta_props)
        TestWMLClientWithHybrid.model_uid = self.client.repository.get_model_uid(published_model)
        TestWMLClientWithHybrid.logger.info("Published model ID:" + str(TestWMLClientWithHybrid.model_uid))
        self.assertIsNotNone(TestWMLClientWithHybrid.model_uid)

    def test_03_load_model(self):
        TestWMLClientWithHybrid.logger.info("Load model from repository: {}".format(TestWMLClientWithHybrid.model_uid))
        self.tf_model = self.client.repository.load(TestWMLClientWithHybrid.model_uid)
        TestWMLClientWithHybrid.logger.debug("SPSS type: {}".format(type(self.tf_model)))
        self.assertTrue(self.tf_model)


    def test_05_get_model_details(self):
        TestWMLClientWithHybrid.logger.info("Get model details")
        details = self.client.repository.get_model_details(TestWMLClientWithHybrid.model_uid)
        self.assertIsNotNone(details)

    def test_11_delete_model(self):
        TestWMLClientWithHybrid.logger.info("Delete model")
        self.client.repository.delete(TestWMLClientWithHybrid.model_uid)
        self.client.spaces.delete(TestWMLClientWithHybrid.space_id)


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import joblib
import logging

from sklearn.base import is_classifier
from sklearn.experimental import enable_hist_gradient_boosting
from sklearn.ensemble import HistGradientBoostingRegressor
from sklearn.ensemble import HistGradientBoostingClassifier
from sklearn.ensemble._hist_gradient_boosting.predictor import TreePredictor
from sklearn.ensemble._hist_gradient_boosting.common import PREDICTOR_RECORD_DTYPE

from sklearn.ensemble._hist_gradient_boosting.loss import BinaryCrossEntropy, \
    CategoricalCrossEntropy, LeastSquares

logger = logging.getLogger(__name__)


def export_sklearn(model, full_path=None):
    """
    Auxillary function to first convert the FL XGBoost Model to Scikit Learn's
    Histogram Gradient Boosting model object, then persist the object as a
    pickled model object. This enables standalone use of the trained model
    without dependency to FL's libraries.

    Parameters/Features that are Not Supported or Transferred:
    - monotonic_cst (in v0.21)
    - warm_start
    - early_stopping
    - scoring
    - validation_fraction
    - n_iter_no_change
    - tol

    :param model: The FL XGBoost model object to convert from.
    :type  model: `XGBFLModel`
    :param full_path: The full absolute path of where the model will persist.
    :type  full_path: `str`
    :return: `None`
    """
    # Initialize Model Object (Parameter Transfer from FL Model to Sklearn)
    if model.loss == 'least_squares':
        export_model = HistGradientBoostingRegressor(
            loss=model.loss,
            learning_rate=model.learning_rate,
            max_iter=model.max_iter,
            max_leaf_nodes=model.max_leaf_nodes,
            max_depth=model.max_depth,
            min_samples_leaf=model.min_samples_leaf,
            l2_regularization=model.l2_regularization,
            max_bins=model.max_bins,
            verbose=model.verbose,
            random_state=model.random_state)
    elif model.loss == 'categorical_crossentropy' or \
            model.loss == 'binary_crossentropy' or \
            model.loss == 'auto':
        export_model = HistGradientBoostingClassifier(
            loss=model.loss,
            learning_rate=model.learning_rate,
            max_iter=model.max_iter,
            max_leaf_nodes=model.max_leaf_nodes,
            max_depth=model.max_depth,
            min_samples_leaf=model.min_samples_leaf,
            l2_regularization=model.l2_regularization,
            max_bins=model.max_bins,
            verbose=model.verbose,
            random_state=model.random_state)

    # Attribute Transfer
    export_model._baseline_prediction = model._baseline_prediction
    export_model.n_trees_per_iteration_ = model.n_trees
    export_model.n_features_ = model.n_features_

    # Model Predictor Object (Deep Object Rebuild)
    export_model._predictors = []
    for i, pred in enumerate(model._predictors):
        export_model._predictors.append([])
        for j, p in enumerate(pred):
            nodes = model._predictors[i][j].nodes.astype(
                PREDICTOR_RECORD_DTYPE).copy()
            export_model._predictors[i].append(TreePredictor(nodes))

    # Model Loss Function
    if hasattr(model, 'classes_'):
        if model.loss == 'binary_crossentropy' or len(model.classes_) == 2:
            export_model.loss_ = BinaryCrossEntropy(None)
        elif model.loss == 'categorical_crossentropy':
            export_model.loss_ = CategoricalCrossEntropy(None)
    elif model.loss == 'least_squares':
        export_model.loss_ = LeastSquares(None)

    # Target Class Encoding (For Classification Models)
    if hasattr(model, 'classes_'):
        export_model.classes_ = model.classes_

    # Perform Model Pickling
    with open(full_path, 'wb') as f:
        joblib.dump(export_model, f)



# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect
from pyspark import SparkContext
from pyspark.ml.pipeline import PipelineModel

from Wrapper import PythonJavaConversions, JavaParams


class IBMSparkPipelineModel(PipelineModel):
    """
    The IBMSparkPipelineModel extends from Spark ML PipelineModel.
    """
    _sc = SparkContext._active_spark_context

    def __init__(self, stages):
        self._jLogger = self._sc._jvm.org.apache.log4j.Logger
        self._logSrcLang = "Py:"
        self.logger = self._jLogger.getLogger(self.__class__.__name__)
        methodname = str(inspect.stack()[0][3])
        logMsg = self._logSrcLang + self.__class__.__name__ + ":" + methodname + ": [Params: " + "stages => " + str(
                stages) + "]"
        self.logger.info(logMsg)

        super(IBMSparkPipelineModel, self).__init__(stages)

    def _to_java(self):
        """
        Transfer this instance to a Java PipelineModel.  Used for ML persistence.

        :return: Java object equivalent to this instance.
        """
        methodname = str(inspect.stack()[0][3])
        logMsg = self._logSrcLang + self.__class__.__name__ + ":" + methodname + ": [Params: None]"
        self.logger.info(logMsg)

        gateway = SparkContext._gateway
        clsname = SparkContext._jvm.org.apache.spark.ml.Transformer
        java_stages = gateway.new_array(clsname, len(self.stages))
        for idx, stage in enumerate(self.stages):
            java_stages[idx] = PythonJavaConversions._to_java_stage(stage)

        _java_obj = \
            JavaParams._new_java_obj("org.apache.spark.ml.IBMSparkPipelineModel", self.uid, java_stages)

        return _java_obj

    def save(self, path):
        """
        Saves a :class:`IBMSparkPipelineModel` object to GPFS.

        :param filename: filename to be saved in GPFS.
        :return: None.
        """
        methodname = str(inspect.stack()[0][3])
        logMsg = self._logSrcLang + self.__class__.__name__ + ":" + methodname + ": [Params: " + " filename => " + str(
                path) + "]"
        self.logger.info(logMsg)

        if not isinstance(path, basestring):
            logMsg = self._logSrcLang + self.__class__.__name__ + ":" + methodname + ": filename should be a " \
                    "basestring, got " + str(type(path))
            self.logger.error(logMsg)
            raise TypeError("filename should be a basestring, got %s" % type(path))

        _jModel = self._to_java()
        _jModel.save(path)

    def saveGpfs(self, path):
        """
        Saves a :class:`IBMSparkPipelineModel` object to GIT.

        :param path: filename to be saved in GIT.
        :return: None.
        """
        methodname = str(inspect.stack()[0][3])
        logMsg = self._logSrcLang + self.__class__.__name__ + ":" + methodname + ": [Params: " + " filename => " + str(
                path) + "]"
        self.logger.info(logMsg)

        _jModel = self._to_java()
        _jModel.writer().saveGpfs(path)

    @classmethod
    def _from_java(cls, java_stage):
        """
        Given a Java PipelineModel, create and return a Python wrapper of it.
        Used for ML persistence.
        """
        # Load information from java_stage to the instance.

        logger = cls._sc._jvm.org.apache.log4j.Logger.getLogger(cls.__name__)
        _logSrcLang = "Py:"

        methodname = str(inspect.stack()[0][3])
        logMsg = _logSrcLang + cls.__name__ + ":" + methodname + ": [Params: " + " java_stage => " + str(
                java_stage) + "]"
        logger.info(logMsg)

        py_stages = [PythonJavaConversions._from_java_stage(s) for s in java_stage.stages()]
        # Create a new instance of this stage.
        py_stage = IBMSparkPipelineModel(py_stages)

        py_stage = PythonJavaConversions._resetUid(py_stage, java_stage.uid())
        return py_stage

    @classmethod
    def load(cls, path):
        """
        Loads a :class:`IBMSparkPipelineModel` object from the saved path in GPFS.

        :param path: filename which was saved in GPFS.
        :return: A :class:`IBMSparkPipelineModel` object.
        """
        logger = cls._sc._jvm.org.apache.log4j.Logger.getLogger(cls.__name__)
        _logSrcLang = "Py:"
        methodname = str(inspect.stack()[0][3])
        logMsg = _logSrcLang + cls.__name__ + ":" + methodname + ": [Params: " + " filename => " + str(path) + "]"
        logger.info(logMsg)

        _jread = PythonJavaConversions._load_java_obj(cls)

        if not isinstance(path, basestring):
            logMsg = _logSrcLang + cls.__name__ + ":" + methodname + ": filename should be a basestring, got " \
                     + str(type(path))
            logger.error(logMsg)
            raise TypeError("filename should be a basestring, got type %s" % type(path))

        java_obj = _jread.load(path)

        return cls._from_java(java_obj)

    @classmethod
    def loadGpfs(cls, path):
        """
        Loads a :class:`IBMSparkPipelineModel` object from the saved path in GPFS.

        :param path: filename which was saved in GIT.
        :return: A :class:`IBMSparkPipelineModel` object.
        """
        logger = cls._sc._jvm.org.apache.log4j.Logger.getLogger(cls.__name__)
        _logSrcLang = "Py:"
        methodname = str(inspect.stack()[0][3])
        logMsg = _logSrcLang + cls.__name__ + ":" + methodname + ": [Params: " + " filename => " + str(path) + "]"
        logger.info(logMsg)

        # return Reader(cls).load(path)

        _jread = PythonJavaConversions._load_java_obj(cls)

        if not isinstance(path, basestring):
            raise TypeError("filename should be a basestring, got type %s" % type(path))

        java_obj = _jread.reader().loadGpfs(path)

        return cls._from_java(java_obj)
